// Note: The license below is based on the template at:
// http://opensource.org/licenses/BSD-3-Clause
// Copyright (C) 2020 Regents of the University of Texas
//All rights reserved.

// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are
// met:

// o Redistributions of source code must retain the above copyright
//   notice, this list of conditions and the following disclaimer.

// o Redistributions in binary form must reproduce the above copyright
//   notice, this list of conditions and the following disclaimer in the
//   documentation and/or other materials provided with the distribution.

// o Neither the name of the copyright holders nor the names of its
//   contributors may be used to endorse or promote products derived
//   from this software without specific prior written permission.

// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

// Original Author(s):
// Mertcan Temel         <mert@utexas.edu>

// DO NOT REMOVE:
// This file is generated by Temel's multiplier generator. Download from https://github.com/temelmertcan/multgen.


// Specification module to help understand what the design implements.
module DOT_Product_WT_SB4_JSkCond_6_11x9_plus_23_22to3_spec (
        input logic [5:0][10:0] IN1,
        input logic [5:0][8:0] IN2,
        input logic [22:0] IN3,
        output logic design_is_correct, // is set to 1 iff the output of DOT_Product_WT_SB4_JSkCond_6_11x9_plus_23_22to3  matches its spec
        output logic [22:3] design_res,
        output logic [22:3] spec_res);
    
    logic [22:0] tmp_spec_res;
    assign tmp_spec_res = (signed'(IN1[0]) * signed'(IN2[0])) + 
		      (signed'(IN1[1]) * signed'(IN2[1])) + 
		      (signed'(IN1[2]) * signed'(IN2[2])) + 
		      (signed'(IN1[3]) * signed'(IN2[3])) + 
		      (signed'(IN1[4]) * signed'(IN2[4])) + 
		      (signed'(IN1[5]) * signed'(IN2[5])) + 
		      signed'(IN3);
    assign spec_res = tmp_spec_res[22:3];
    DOT_Product_WT_SB4_JSkCond_6_11x9_plus_23_22to3 dot_product(IN1, IN2, IN3, design_res);
    assign design_is_correct = ((spec_res == design_res) ? 1 : 0);
    
endmodule

module WT_SB4_11x9(
        input logic [10:0] IN1,
        input logic [8:0] IN2,
        output logic [22:0] result0,
        output logic [22:0] result1);
    
    
// Creating Partial Products 

    wire [10:0] mult = IN1;
    wire [8:0] mcand = IN2;
    wire [9:0] mcand_1x;
    wire [9:0] mcand_2x;
    assign mcand_1x = {{1{mcand[8]}},  mcand};
    assign mcand_2x = {{0{mcand[8]}},  mcand, 1'b0};
    
    // Booth Radix-4 Partial Products. Multiplier selectors: mult[1] mult[0] 1'b0
    wire logic select_0_0, select_e_0, select_2x_0, tcomp0, select_ne_0, select_n2x_0;
    assign select_0_0 =  &{mult[1], mult[0], 1'b0} | ~|{mult[1], mult[0], 1'b0};
    assign select_e_0 = ((~ mult[1]) & (mult[0] ^ 1'b0));
    assign select_ne_0 = mult[1] &  (mult[0] ^ 1'b0);
    assign select_2x_0 = (~ mult[1]) & mult[0] & 1'b0;
    assign select_n2x_0 = mult[1] & (~ mult[0]) & (~ 1'b0);
    reg [9:0] pp_0;
    always @(*) begin
       case (1'b1)
          select_0_0   : pp_0 = 0; 
          select_e_0   : pp_0 = mcand_1x; 
          select_2x_0  : pp_0 = mcand_2x; 
          select_n2x_0 : pp_0 = (~ mcand_2x); 
          select_ne_0  : pp_0 = (~ mcand_1x); 
       endcase 
       pp_0[9] = ~pp_0[9]; // flip the MSB 
    end
    assign tcomp0 =  select_n2x_0 | select_ne_0;
    
    // Booth Radix-4 Partial Products. Multiplier selectors: mult[3] mult[2] mult[1]
    wire logic select_0_1, select_e_1, select_2x_1, tcomp1, select_ne_1, select_n2x_1;
    assign select_0_1 =  &{mult[3], mult[2], mult[1]} | ~|{mult[3], mult[2], mult[1]};
    assign select_e_1 = ((~ mult[3]) & (mult[2] ^ mult[1]));
    assign select_ne_1 = mult[3] &  (mult[2] ^ mult[1]);
    assign select_2x_1 = (~ mult[3]) & mult[2] & mult[1];
    assign select_n2x_1 = mult[3] & (~ mult[2]) & (~ mult[1]);
    reg [9:0] pp_1;
    always @(*) begin
       case (1'b1)
          select_0_1   : pp_1 = 0; 
          select_e_1   : pp_1 = mcand_1x; 
          select_2x_1  : pp_1 = mcand_2x; 
          select_n2x_1 : pp_1 = (~ mcand_2x); 
          select_ne_1  : pp_1 = (~ mcand_1x); 
       endcase 
       pp_1[9] = ~pp_1[9]; // flip the MSB 
    end
    assign tcomp1 =  select_n2x_1 | select_ne_1;
    
    // Booth Radix-4 Partial Products. Multiplier selectors: mult[5] mult[4] mult[3]
    wire logic select_0_2, select_e_2, select_2x_2, tcomp2, select_ne_2, select_n2x_2;
    assign select_0_2 =  &{mult[5], mult[4], mult[3]} | ~|{mult[5], mult[4], mult[3]};
    assign select_e_2 = ((~ mult[5]) & (mult[4] ^ mult[3]));
    assign select_ne_2 = mult[5] &  (mult[4] ^ mult[3]);
    assign select_2x_2 = (~ mult[5]) & mult[4] & mult[3];
    assign select_n2x_2 = mult[5] & (~ mult[4]) & (~ mult[3]);
    reg [9:0] pp_2;
    always @(*) begin
       case (1'b1)
          select_0_2   : pp_2 = 0; 
          select_e_2   : pp_2 = mcand_1x; 
          select_2x_2  : pp_2 = mcand_2x; 
          select_n2x_2 : pp_2 = (~ mcand_2x); 
          select_ne_2  : pp_2 = (~ mcand_1x); 
       endcase 
       pp_2[9] = ~pp_2[9]; // flip the MSB 
    end
    assign tcomp2 =  select_n2x_2 | select_ne_2;
    
    // Booth Radix-4 Partial Products. Multiplier selectors: mult[7] mult[6] mult[5]
    wire logic select_0_3, select_e_3, select_2x_3, tcomp3, select_ne_3, select_n2x_3;
    assign select_0_3 =  &{mult[7], mult[6], mult[5]} | ~|{mult[7], mult[6], mult[5]};
    assign select_e_3 = ((~ mult[7]) & (mult[6] ^ mult[5]));
    assign select_ne_3 = mult[7] &  (mult[6] ^ mult[5]);
    assign select_2x_3 = (~ mult[7]) & mult[6] & mult[5];
    assign select_n2x_3 = mult[7] & (~ mult[6]) & (~ mult[5]);
    reg [9:0] pp_3;
    always @(*) begin
       case (1'b1)
          select_0_3   : pp_3 = 0; 
          select_e_3   : pp_3 = mcand_1x; 
          select_2x_3  : pp_3 = mcand_2x; 
          select_n2x_3 : pp_3 = (~ mcand_2x); 
          select_ne_3  : pp_3 = (~ mcand_1x); 
       endcase 
       pp_3[9] = ~pp_3[9]; // flip the MSB 
    end
    assign tcomp3 =  select_n2x_3 | select_ne_3;
    
    // Booth Radix-4 Partial Products. Multiplier selectors: mult[9] mult[8] mult[7]
    wire logic select_0_4, select_e_4, select_2x_4, tcomp4, select_ne_4, select_n2x_4;
    assign select_0_4 =  &{mult[9], mult[8], mult[7]} | ~|{mult[9], mult[8], mult[7]};
    assign select_e_4 = ((~ mult[9]) & (mult[8] ^ mult[7]));
    assign select_ne_4 = mult[9] &  (mult[8] ^ mult[7]);
    assign select_2x_4 = (~ mult[9]) & mult[8] & mult[7];
    assign select_n2x_4 = mult[9] & (~ mult[8]) & (~ mult[7]);
    reg [9:0] pp_4;
    always @(*) begin
       case (1'b1)
          select_0_4   : pp_4 = 0; 
          select_e_4   : pp_4 = mcand_1x; 
          select_2x_4  : pp_4 = mcand_2x; 
          select_n2x_4 : pp_4 = (~ mcand_2x); 
          select_ne_4  : pp_4 = (~ mcand_1x); 
       endcase 
       pp_4[9] = ~pp_4[9]; // flip the MSB 
    end
    assign tcomp4 =  select_n2x_4 | select_ne_4;
    
    // Booth Radix-4 Partial Products. Multiplier selectors: mult[10] mult[10] mult[9]
    wire logic select_0_5, select_e_5, select_2x_5, tcomp5, select_ne_5, select_n2x_5;
    assign select_0_5 =  &{mult[10], mult[10], mult[9]} | ~|{mult[10], mult[10], mult[9]};
    assign select_e_5 = ((~ mult[10]) & (mult[10] ^ mult[9]));
    assign select_ne_5 = mult[10] &  (mult[10] ^ mult[9]);
    assign select_2x_5 = (~ mult[10]) & mult[10] & mult[9];
    assign select_n2x_5 = mult[10] & (~ mult[10]) & (~ mult[9]);
    reg [9:0] pp_5;
    always @(*) begin
       case (1'b1)
          select_0_5   : pp_5 = 0; 
          select_e_5   : pp_5 = mcand_1x; 
          select_2x_5  : pp_5 = mcand_2x; 
          select_n2x_5 : pp_5 = (~ mcand_2x); 
          select_ne_5  : pp_5 = (~ mcand_1x); 
       endcase 
       pp_5[9] = ~pp_5[9]; // flip the MSB 
    end
    assign tcomp5 =  select_n2x_5 | select_ne_5;
    
    // The values to be summed in the summation tree, from LSB (left) to MSB:
     // pp_0[0] pp_0[1] pp_0[2] pp_0[3] pp_0[4] pp_0[5] pp_0[6] pp_0[7] pp_0[8] pp_0[9]   --      --      --      --      --      --      --      --      --      --    
     //   --      --    pp_1[0] pp_1[1] pp_1[2] pp_1[3] pp_1[4] pp_1[5] pp_1[6] pp_1[7] pp_1[8] pp_1[9]   --      --      --      --      --      --      --      --    
     //   --      --      --      --    pp_2[0] pp_2[1] pp_2[2] pp_2[3] pp_2[4] pp_2[5] pp_2[6] pp_2[7] pp_2[8] pp_2[9]   --      --      --      --      --      --    
     //   --      --      --      --      --      --    pp_3[0] pp_3[1] pp_3[2] pp_3[3] pp_3[4] pp_3[5] pp_3[6] pp_3[7] pp_3[8] pp_3[9]   --      --      --      --    
     //   --      --      --      --      --      --      --      --    pp_4[0] pp_4[1] pp_4[2] pp_4[3] pp_4[4] pp_4[5] pp_4[6] pp_4[7] pp_4[8] pp_4[9]   --      --    
     //   --      --      --      --      --      --      --      --      --      --    pp_5[0] pp_5[1] pp_5[2] pp_5[3] pp_5[4] pp_5[5] pp_5[6] pp_5[7] pp_5[8] pp_5[9] 
     // tcomp0    --    tcomp1    --    tcomp2    --    tcomp3    --    tcomp4    --    tcomp5    --      --      --      --      --      --      --      --      --    
     //   --      --      --      --      --      --      --      --      --      --      --      --      --      --      --      --      --      --      --      --    
    
// Creating Summation Tree 

    
    // Wallace Summation Stage 1
    logic s0 ,c0;
    ha ha0 (pp_0[0], tcomp0, s0, c0);
    logic s1 ,c1; 
    fa fa1 (pp_0[2], pp_1[0], tcomp1, s1, c1);
    logic s2 ,c2;
    ha ha2 (pp_0[3], pp_1[1], s2, c2);
    logic s3 ,c3; 
    fa fa3 (pp_0[4], pp_1[2], pp_2[0], s3, c3);
    logic s4 ,c4; 
    fa fa4 (pp_0[5], pp_1[3], pp_2[1], s4, c4);
    logic s5 ,c5; 
    fa fa5 (pp_0[6], pp_1[4], pp_2[2], s5, c5);
    logic s6 ,c6;
    ha ha6 (pp_3[0], tcomp3, s6, c6);
    logic s7 ,c7; 
    fa fa7 (pp_0[7], pp_1[5], pp_2[3], s7, c7);
    logic s8 ,c8; 
    fa fa8 (pp_0[8], pp_1[6], pp_2[4], s8, c8);
    logic s9 ,c9; 
    fa fa9 (pp_3[2], pp_4[0], tcomp4, s9, c9);
    logic s10 ,c10; 
    fa fa10 (pp_0[9], pp_1[7], pp_2[5], s10, c10);
    logic s11 ,c11;
    ha ha11 (pp_3[3], pp_4[1], s11, c11);
    logic s12 ,c12; 
    fa fa12 (pp_1[8], pp_2[6], pp_3[4], s12, c12);
    logic s13 ,c13; 
    fa fa13 (pp_4[2], pp_5[0], tcomp5, s13, c13);
    logic s14 ,c14; 
    fa fa14 (pp_1[9], pp_2[7], pp_3[5], s14, c14);
    logic s15 ,c15;
    ha ha15 (pp_4[3], pp_5[1], s15, c15);
    logic s16 ,c16; 
    fa fa16 (pp_2[8], pp_3[6], pp_4[4], s16, c16);
    logic s17 ,c17; 
    fa fa17 (pp_2[9], pp_3[7], pp_4[5], s17, c17);
    logic s18 ,c18; 
    fa fa18 (pp_3[8], pp_4[6], pp_5[4], s18, c18);
    logic s19 ,c19; 
    fa fa19 (pp_3[9], pp_4[7], pp_5[5], s19, c19);
    logic s20 ,c20;
    ha ha20 (pp_4[8], pp_5[6], s20, c20);
    logic s21 ,c21;
    ha ha21 (pp_4[9], pp_5[7], s21, c21);
    
    // Wallace Summation Stage 2
    logic s22 ,c22;
    ha ha22 (pp_0[1], c0, s22, c22);
    logic s23 ,c23;
    ha ha23 (c1, s2, s23, c23);
    logic s24 ,c24; 
    fa fa24 (tcomp2, c2, s3, s24, c24);
    logic s25 ,c25;
    ha ha25 (c3, s4, s25, c25);
    logic s26 ,c26; 
    fa fa26 (c4, s5, s6, s26, c26);
    logic s27 ,c27; 
    fa fa27 (pp_3[1], c5, c6, s27, c27);
    logic s28 ,c28; 
    fa fa28 (c7, s8, s9, s28, c28);
    logic s29 ,c29; 
    fa fa29 (c8, c9, s10, s29, c29);
    logic s30 ,c30; 
    fa fa30 (c10, c11, s12, s30, c30);
    logic s31 ,c31; 
    fa fa31 (c12, c13, s14, s31, c31);
    logic s32 ,c32; 
    fa fa32 (pp_5[2], c14, c15, s32, c32);
    logic s33 ,c33; 
    fa fa33 (pp_5[3], c16, s17, s33, c33);
    logic s34 ,c34;
    ha ha34 (c17, s18, s34, c34);
    logic s35 ,c35;
    ha ha35 (c18, s19, s35, c35);
    logic s36 ,c36;
    ha ha36 (c19, s20, s36, c36);
    logic s37 ,c37;
    ha ha37 (c20, s21, s37, c37);
    logic s38 ,c38;
    ha ha38 (pp_5[8], c21, s38, c38);
    
    // Wallace Summation Stage 3
    logic s39 ,c39;
    ha ha39 (s1, c22, s39, c39);
    logic s40 ,c40;
    ha ha40 (c23, s24, s40, c40);
    logic s41 ,c41;
    ha ha41 (c24, s25, s41, c41);
    logic s42 ,c42;
    ha ha42 (c25, s26, s42, c42);
    logic s43 ,c43; 
    fa fa43 (s7, c26, s27, s43, c43);
    logic s44 ,c44;
    ha ha44 (c27, s28, s44, c44);
    logic s45 ,c45; 
    fa fa45 (s11, c28, s29, s45, c45);
    logic s46 ,c46; 
    fa fa46 (s13, c29, s30, s46, c46);
    logic s47 ,c47; 
    fa fa47 (s15, c30, s31, s47, c47);
    logic s48 ,c48; 
    fa fa48 (s16, c31, s32, s48, c48);
    logic s49 ,c49;
    ha ha49 (c32, s33, s49, c49);
    logic s50 ,c50;
    ha ha50 (c33, s34, s50, c50);
    logic s51 ,c51;
    ha ha51 (c34, s35, s51, c51);
    logic s52 ,c52;
    ha ha52 (c35, s36, s52, c52);
    logic s53 ,c53;
    ha ha53 (c36, s37, s53, c53);
    logic s54 ,c54;
    ha ha54 (c37, s38, s54, c54);
    logic s55 ,c55;
    ha ha55 (pp_5[9], c38, s55, c55);
    
    assign result0[0] = s0;
    assign result1[0] = 1'b0;
    assign result0[1] = s22;
    assign result1[1] = 1'b0;
    assign result0[2] = s39;
    assign result1[2] = 1'b0;
    
    assign result0[20:3] = {c55, c54, c53, c52, c51, c50, c49, c48, c47, c46, c45, c44, c43, c42, c41, c40, s40, s23 };
    assign result1[20:3] = {1'b0, s55, s54, s53, s52, s51, s50, s49, s48, s47, s46, s45, s44, s43, s42, s41, 1'b0, c39 };
    assign result0[21] = 1'b0;
    assign result1[21] = 1'b0;
    assign result0[22] = 1'b0;
    assign result1[22] = 1'b0;
endmodule



module DOT_Product_WT_SB4_JSkCond_6_11x9_plus_23_22to3(
        input logic [5:0][10:0] IN1,
        input logic [5:0][8:0] IN2,
        input logic [22:0] IN3,
        output logic [22:3] result);
    
    wire logic [22:0] m0_0;
    wire logic [22:0] m0_1;
    wire logic [22:0] m1_0;
    wire logic [22:0] m1_1;
    wire logic [22:0] m2_0;
    wire logic [22:0] m2_1;
    wire logic [22:0] m3_0;
    wire logic [22:0] m3_1;
    wire logic [22:0] m4_0;
    wire logic [22:0] m4_1;
    wire logic [22:0] m5_0;
    wire logic [22:0] m5_1;
    
    WT_SB4_11x9 m0 (IN1[0][10:0], IN2[0][8:0], m0_0, m0_1);
    WT_SB4_11x9 m1 (IN1[1][10:0], IN2[1][8:0], m1_0, m1_1);
    WT_SB4_11x9 m2 (IN1[2][10:0], IN2[2][8:0], m2_0, m2_1);
    WT_SB4_11x9 m3 (IN1[3][10:0], IN2[3][8:0], m3_0, m3_1);
    WT_SB4_11x9 m4 (IN1[4][10:0], IN2[4][8:0], m4_0, m4_1);
    WT_SB4_11x9 m5 (IN1[5][10:0], IN2[5][8:0], m5_0, m5_1);
    
    // The values to be summed in the summation tree, from LSB (left) to MSB:
     // m0_0[0]  m0_0[1]  m0_0[2]  m0_0[3]  m0_0[4]  m0_0[5]  m0_0[6]  m0_0[7]  m0_0[8]  m0_0[9]  m0_0[10] m0_0[11] m0_0[12] m0_0[13] m0_0[14] m0_0[15] m0_0[16] m0_0[17] m0_0[18] m0_0[19] m0_0[20]   --       --     
     //   --       --       --     m0_1[3]    --     m0_1[5]  m0_1[6]  m0_1[7]  m0_1[8]  m0_1[9]  m0_1[10] m0_1[11] m0_1[12] m0_1[13] m0_1[14] m0_1[15] m0_1[16] m0_1[17] m0_1[18] m0_1[19]   --       --       --     
     // m1_0[0]  m1_0[1]  m1_0[2]  m1_0[3]  m1_0[4]  m1_0[5]  m1_0[6]  m1_0[7]  m1_0[8]  m1_0[9]  m1_0[10] m1_0[11] m1_0[12] m1_0[13] m1_0[14] m1_0[15] m1_0[16] m1_0[17] m1_0[18] m1_0[19] m1_0[20]   --       --     
     //   --       --       --     m1_1[3]    --     m1_1[5]  m1_1[6]  m1_1[7]  m1_1[8]  m1_1[9]  m1_1[10] m1_1[11] m1_1[12] m1_1[13] m1_1[14] m1_1[15] m1_1[16] m1_1[17] m1_1[18] m1_1[19]   --       --       --     
     // m2_0[0]  m2_0[1]  m2_0[2]  m2_0[3]  m2_0[4]  m2_0[5]  m2_0[6]  m2_0[7]  m2_0[8]  m2_0[9]  m2_0[10] m2_0[11] m2_0[12] m2_0[13] m2_0[14] m2_0[15] m2_0[16] m2_0[17] m2_0[18] m2_0[19] m2_0[20]   --       --     
     //   --       --       --     m2_1[3]    --     m2_1[5]  m2_1[6]  m2_1[7]  m2_1[8]  m2_1[9]  m2_1[10] m2_1[11] m2_1[12] m2_1[13] m2_1[14] m2_1[15] m2_1[16] m2_1[17] m2_1[18] m2_1[19]   --       --       --     
     // m3_0[0]  m3_0[1]  m3_0[2]  m3_0[3]  m3_0[4]  m3_0[5]  m3_0[6]  m3_0[7]  m3_0[8]  m3_0[9]  m3_0[10] m3_0[11] m3_0[12] m3_0[13] m3_0[14] m3_0[15] m3_0[16] m3_0[17] m3_0[18] m3_0[19] m3_0[20]   --       --     
     //   --       --       --     m3_1[3]    --     m3_1[5]  m3_1[6]  m3_1[7]  m3_1[8]  m3_1[9]  m3_1[10] m3_1[11] m3_1[12] m3_1[13] m3_1[14] m3_1[15] m3_1[16] m3_1[17] m3_1[18] m3_1[19]   --       --       --     
     // m4_0[0]  m4_0[1]  m4_0[2]  m4_0[3]  m4_0[4]  m4_0[5]  m4_0[6]  m4_0[7]  m4_0[8]  m4_0[9]  m4_0[10] m4_0[11] m4_0[12] m4_0[13] m4_0[14] m4_0[15] m4_0[16] m4_0[17] m4_0[18] m4_0[19] m4_0[20]   --       --     
     //   --       --       --     m4_1[3]    --     m4_1[5]  m4_1[6]  m4_1[7]  m4_1[8]  m4_1[9]  m4_1[10] m4_1[11] m4_1[12] m4_1[13] m4_1[14] m4_1[15] m4_1[16] m4_1[17] m4_1[18] m4_1[19]   --       --       --     
     // m5_0[0]  m5_0[1]  m5_0[2]  m5_0[3]  m5_0[4]  m5_0[5]  m5_0[6]  m5_0[7]  m5_0[8]  m5_0[9]  m5_0[10] m5_0[11] m5_0[12] m5_0[13] m5_0[14] m5_0[15] m5_0[16] m5_0[17] m5_0[18] m5_0[19] m5_0[20]   --       --     
     //   --       --       --     m5_1[3]    --     m5_1[5]  m5_1[6]  m5_1[7]  m5_1[8]  m5_1[9]  m5_1[10] m5_1[11] m5_1[12] m5_1[13] m5_1[14] m5_1[15] m5_1[16] m5_1[17] m5_1[18] m5_1[19]   --       --       --     
     // IN3[0]   IN3[1]   IN3[2]   IN3[3]   IN3[4]   IN3[5]   IN3[6]   IN3[7]   IN3[8]   IN3[9]   IN3[10]  IN3[11]  IN3[12]  IN3[13]  IN3[14]  IN3[15]  IN3[16]  IN3[17]  IN3[18]  IN3[19]  IN3[20]  IN3[21]  ~IN3[22] 
     //   --       --       --       --       --       --       --       --       --       --     1'b1       --       --       --       --       --       --       --       --       --       --       --       --     
    
    // Wallace Summation Stage 1
    logic s0 ,c0; 
    fa fa0 (m0_0[0], m1_0[0], m2_0[0], s0, c0);
    logic s1 ,c1; 
    fa fa1 (m3_0[0], m4_0[0], m5_0[0], s1, c1);
    logic s2 ,c2; 
    fa fa2 (m0_0[1], m1_0[1], m2_0[1], s2, c2);
    logic s3 ,c3; 
    fa fa3 (m3_0[1], m4_0[1], m5_0[1], s3, c3);
    logic s4 ,c4; 
    fa fa4 (m0_0[2], m1_0[2], m2_0[2], s4, c4);
    logic s5 ,c5; 
    fa fa5 (m3_0[2], m4_0[2], m5_0[2], s5, c5);
    logic s6 ,c6; 
    fa fa6 (m0_0[3], m0_1[3], m1_0[3], s6, c6);
    logic s7 ,c7; 
    fa fa7 (m1_1[3], m2_0[3], m2_1[3], s7, c7);
    logic s8 ,c8; 
    fa fa8 (m3_0[3], m3_1[3], m4_0[3], s8, c8);
    logic s9 ,c9; 
    fa fa9 (m4_1[3], m5_0[3], m5_1[3], s9, c9);
    logic s10 ,c10; 
    fa fa10 (m0_0[4], m1_0[4], m2_0[4], s10, c10);
    logic s11 ,c11; 
    fa fa11 (m3_0[4], m4_0[4], m5_0[4], s11, c11);
    logic s12 ,c12; 
    fa fa12 (m0_0[5], m0_1[5], m1_0[5], s12, c12);
    logic s13 ,c13; 
    fa fa13 (m1_1[5], m2_0[5], m2_1[5], s13, c13);
    logic s14 ,c14; 
    fa fa14 (m3_0[5], m3_1[5], m4_0[5], s14, c14);
    logic s15 ,c15; 
    fa fa15 (m4_1[5], m5_0[5], m5_1[5], s15, c15);
    logic s16 ,c16; 
    fa fa16 (m0_0[6], m0_1[6], m1_0[6], s16, c16);
    logic s17 ,c17; 
    fa fa17 (m1_1[6], m2_0[6], m2_1[6], s17, c17);
    logic s18 ,c18; 
    fa fa18 (m3_0[6], m3_1[6], m4_0[6], s18, c18);
    logic s19 ,c19; 
    fa fa19 (m4_1[6], m5_0[6], m5_1[6], s19, c19);
    logic s20 ,c20; 
    fa fa20 (m0_0[7], m0_1[7], m1_0[7], s20, c20);
    logic s21 ,c21; 
    fa fa21 (m1_1[7], m2_0[7], m2_1[7], s21, c21);
    logic s22 ,c22; 
    fa fa22 (m3_0[7], m3_1[7], m4_0[7], s22, c22);
    logic s23 ,c23; 
    fa fa23 (m4_1[7], m5_0[7], m5_1[7], s23, c23);
    logic s24 ,c24; 
    fa fa24 (m0_0[8], m0_1[8], m1_0[8], s24, c24);
    logic s25 ,c25; 
    fa fa25 (m1_1[8], m2_0[8], m2_1[8], s25, c25);
    logic s26 ,c26; 
    fa fa26 (m3_0[8], m3_1[8], m4_0[8], s26, c26);
    logic s27 ,c27; 
    fa fa27 (m4_1[8], m5_0[8], m5_1[8], s27, c27);
    logic s28 ,c28; 
    fa fa28 (m0_0[9], m0_1[9], m1_0[9], s28, c28);
    logic s29 ,c29; 
    fa fa29 (m1_1[9], m2_0[9], m2_1[9], s29, c29);
    logic s30 ,c30; 
    fa fa30 (m3_0[9], m3_1[9], m4_0[9], s30, c30);
    logic s31 ,c31; 
    fa fa31 (m4_1[9], m5_0[9], m5_1[9], s31, c31);
    logic s32 ,c32; 
    fa fa32 (m0_0[10], m0_1[10], m1_0[10], s32, c32);
    logic s33 ,c33; 
    fa fa33 (m1_1[10], m2_0[10], m2_1[10], s33, c33);
    logic s34 ,c34; 
    fa fa34 (m3_0[10], m3_1[10], m4_0[10], s34, c34);
    logic s35 ,c35; 
    fa fa35 (m4_1[10], m5_0[10], m5_1[10], s35, c35);
    logic s36 ,c36;
    ha ha36 (IN3[10], 1'b1, s36, c36);
    logic s37 ,c37; 
    fa fa37 (m0_0[11], m0_1[11], m1_0[11], s37, c37);
    logic s38 ,c38; 
    fa fa38 (m1_1[11], m2_0[11], m2_1[11], s38, c38);
    logic s39 ,c39; 
    fa fa39 (m3_0[11], m3_1[11], m4_0[11], s39, c39);
    logic s40 ,c40; 
    fa fa40 (m4_1[11], m5_0[11], m5_1[11], s40, c40);
    logic s41 ,c41; 
    fa fa41 (m0_0[12], m0_1[12], m1_0[12], s41, c41);
    logic s42 ,c42; 
    fa fa42 (m1_1[12], m2_0[12], m2_1[12], s42, c42);
    logic s43 ,c43; 
    fa fa43 (m3_0[12], m3_1[12], m4_0[12], s43, c43);
    logic s44 ,c44; 
    fa fa44 (m4_1[12], m5_0[12], m5_1[12], s44, c44);
    logic s45 ,c45; 
    fa fa45 (m0_0[13], m0_1[13], m1_0[13], s45, c45);
    logic s46 ,c46; 
    fa fa46 (m1_1[13], m2_0[13], m2_1[13], s46, c46);
    logic s47 ,c47; 
    fa fa47 (m3_0[13], m3_1[13], m4_0[13], s47, c47);
    logic s48 ,c48; 
    fa fa48 (m4_1[13], m5_0[13], m5_1[13], s48, c48);
    logic s49 ,c49; 
    fa fa49 (m0_0[14], m0_1[14], m1_0[14], s49, c49);
    logic s50 ,c50; 
    fa fa50 (m1_1[14], m2_0[14], m2_1[14], s50, c50);
    logic s51 ,c51; 
    fa fa51 (m3_0[14], m3_1[14], m4_0[14], s51, c51);
    logic s52 ,c52; 
    fa fa52 (m4_1[14], m5_0[14], m5_1[14], s52, c52);
    logic s53 ,c53; 
    fa fa53 (m0_0[15], m0_1[15], m1_0[15], s53, c53);
    logic s54 ,c54; 
    fa fa54 (m1_1[15], m2_0[15], m2_1[15], s54, c54);
    logic s55 ,c55; 
    fa fa55 (m3_0[15], m3_1[15], m4_0[15], s55, c55);
    logic s56 ,c56; 
    fa fa56 (m4_1[15], m5_0[15], m5_1[15], s56, c56);
    logic s57 ,c57; 
    fa fa57 (m0_0[16], m0_1[16], m1_0[16], s57, c57);
    logic s58 ,c58; 
    fa fa58 (m1_1[16], m2_0[16], m2_1[16], s58, c58);
    logic s59 ,c59; 
    fa fa59 (m3_0[16], m3_1[16], m4_0[16], s59, c59);
    logic s60 ,c60; 
    fa fa60 (m4_1[16], m5_0[16], m5_1[16], s60, c60);
    logic s61 ,c61; 
    fa fa61 (m0_0[17], m0_1[17], m1_0[17], s61, c61);
    logic s62 ,c62; 
    fa fa62 (m1_1[17], m2_0[17], m2_1[17], s62, c62);
    logic s63 ,c63; 
    fa fa63 (m3_0[17], m3_1[17], m4_0[17], s63, c63);
    logic s64 ,c64; 
    fa fa64 (m4_1[17], m5_0[17], m5_1[17], s64, c64);
    logic s65 ,c65; 
    fa fa65 (m0_0[18], m0_1[18], m1_0[18], s65, c65);
    logic s66 ,c66; 
    fa fa66 (m1_1[18], m2_0[18], m2_1[18], s66, c66);
    logic s67 ,c67; 
    fa fa67 (m3_0[18], m3_1[18], m4_0[18], s67, c67);
    logic s68 ,c68; 
    fa fa68 (m4_1[18], m5_0[18], m5_1[18], s68, c68);
    logic s69 ,c69; 
    fa fa69 (m0_0[19], m0_1[19], m1_0[19], s69, c69);
    logic s70 ,c70; 
    fa fa70 (m1_1[19], m2_0[19], m2_1[19], s70, c70);
    logic s71 ,c71; 
    fa fa71 (m3_0[19], m3_1[19], m4_0[19], s71, c71);
    logic s72 ,c72; 
    fa fa72 (m4_1[19], m5_0[19], m5_1[19], s72, c72);
    logic s73 ,c73; 
    fa fa73 (m0_0[20], m1_0[20], m2_0[20], s73, c73);
    logic s74 ,c74; 
    fa fa74 (m3_0[20], m4_0[20], m5_0[20], s74, c74);
    
    // Wallace Summation Stage 2
    logic s75 ,c75; 
    fa fa75 (IN3[0], s0, s1, s75, c75);
    logic s76 ,c76; 
    fa fa76 (IN3[1], c0, c1, s76, c76);
    logic s77 ,c77;
    ha ha77 (s2, s3, s77, c77);
    logic s78 ,c78; 
    fa fa78 (IN3[2], c2, c3, s78, c78);
    logic s79 ,c79;
    ha ha79 (s4, s5, s79, c79);
    logic s80 ,c80; 
    fa fa80 (IN3[3], c4, c5, s80, c80);
    logic s81 ,c81; 
    fa fa81 (s6, s7, s8, s81, c81);
    logic s82 ,c82; 
    fa fa82 (IN3[4], c6, c7, s82, c82);
    logic s83 ,c83; 
    fa fa83 (c8, c9, s10, s83, c83);
    logic s84 ,c84; 
    fa fa84 (IN3[5], c10, c11, s84, c84);
    logic s85 ,c85; 
    fa fa85 (s12, s13, s14, s85, c85);
    logic s86 ,c86; 
    fa fa86 (IN3[6], c12, c13, s86, c86);
    logic s87 ,c87; 
    fa fa87 (c14, c15, s16, s87, c87);
    logic s88 ,c88; 
    fa fa88 (s17, s18, s19, s88, c88);
    logic s89 ,c89; 
    fa fa89 (IN3[7], c16, c17, s89, c89);
    logic s90 ,c90; 
    fa fa90 (c18, c19, s20, s90, c90);
    logic s91 ,c91; 
    fa fa91 (s21, s22, s23, s91, c91);
    logic s92 ,c92; 
    fa fa92 (IN3[8], c20, c21, s92, c92);
    logic s93 ,c93; 
    fa fa93 (c22, c23, s24, s93, c93);
    logic s94 ,c94; 
    fa fa94 (s25, s26, s27, s94, c94);
    logic s95 ,c95; 
    fa fa95 (IN3[9], c24, c25, s95, c95);
    logic s96 ,c96; 
    fa fa96 (c26, c27, s28, s96, c96);
    logic s97 ,c97; 
    fa fa97 (s29, s30, s31, s97, c97);
    logic s98 ,c98; 
    fa fa98 (c28, c29, c30, s98, c98);
    logic s99 ,c99; 
    fa fa99 (c31, s32, s33, s99, c99);
    logic s100 ,c100; 
    fa fa100 (s34, s35, s36, s100, c100);
    logic s101 ,c101; 
    fa fa101 (IN3[11], c32, c33, s101, c101);
    logic s102 ,c102; 
    fa fa102 (c34, c35, c36, s102, c102);
    logic s103 ,c103; 
    fa fa103 (s37, s38, s39, s103, c103);
    logic s104 ,c104; 
    fa fa104 (IN3[12], c37, c38, s104, c104);
    logic s105 ,c105; 
    fa fa105 (c39, c40, s41, s105, c105);
    logic s106 ,c106; 
    fa fa106 (s42, s43, s44, s106, c106);
    logic s107 ,c107; 
    fa fa107 (IN3[13], c41, c42, s107, c107);
    logic s108 ,c108; 
    fa fa108 (c43, c44, s45, s108, c108);
    logic s109 ,c109; 
    fa fa109 (s46, s47, s48, s109, c109);
    logic s110 ,c110; 
    fa fa110 (IN3[14], c45, c46, s110, c110);
    logic s111 ,c111; 
    fa fa111 (c47, c48, s49, s111, c111);
    logic s112 ,c112; 
    fa fa112 (s50, s51, s52, s112, c112);
    logic s113 ,c113; 
    fa fa113 (IN3[15], c49, c50, s113, c113);
    logic s114 ,c114; 
    fa fa114 (c51, c52, s53, s114, c114);
    logic s115 ,c115; 
    fa fa115 (s54, s55, s56, s115, c115);
    logic s116 ,c116; 
    fa fa116 (IN3[16], c53, c54, s116, c116);
    logic s117 ,c117; 
    fa fa117 (c55, c56, s57, s117, c117);
    logic s118 ,c118; 
    fa fa118 (s58, s59, s60, s118, c118);
    logic s119 ,c119; 
    fa fa119 (IN3[17], c57, c58, s119, c119);
    logic s120 ,c120; 
    fa fa120 (c59, c60, s61, s120, c120);
    logic s121 ,c121; 
    fa fa121 (s62, s63, s64, s121, c121);
    logic s122 ,c122; 
    fa fa122 (IN3[18], c61, c62, s122, c122);
    logic s123 ,c123; 
    fa fa123 (c63, c64, s65, s123, c123);
    logic s124 ,c124; 
    fa fa124 (s66, s67, s68, s124, c124);
    logic s125 ,c125; 
    fa fa125 (IN3[19], c65, c66, s125, c125);
    logic s126 ,c126; 
    fa fa126 (c67, c68, s69, s126, c126);
    logic s127 ,c127; 
    fa fa127 (s70, s71, s72, s127, c127);
    logic s128 ,c128; 
    fa fa128 (IN3[20], c69, c70, s128, c128);
    logic s129 ,c129; 
    fa fa129 (c71, c72, s73, s129, c129);
    logic s130 ,c130; 
    fa fa130 (IN3[21], c73, c74, s130, c130);
    
    // Wallace Summation Stage 3
    logic s131 ,c131; 
    fa fa131 (c75, s76, s77, s131, c131);
    logic s132 ,c132; 
    fa fa132 (c76, c77, s78, s132, c132);
    logic s133 ,c133; 
    fa fa133 (s9, c78, c79, s133, c133);
    logic s134 ,c134;
    ha ha134 (s80, s81, s134, c134);
    logic s135 ,c135; 
    fa fa135 (s11, c80, c81, s135, c135);
    logic s136 ,c136;
    ha ha136 (s82, s83, s136, c136);
    logic s137 ,c137; 
    fa fa137 (s15, c82, c83, s137, c137);
    logic s138 ,c138;
    ha ha138 (s84, s85, s138, c138);
    logic s139 ,c139; 
    fa fa139 (c84, c85, s86, s139, c139);
    logic s140 ,c140;
    ha ha140 (s87, s88, s140, c140);
    logic s141 ,c141; 
    fa fa141 (c86, c87, c88, s141, c141);
    logic s142 ,c142; 
    fa fa142 (s89, s90, s91, s142, c142);
    logic s143 ,c143; 
    fa fa143 (c89, c90, c91, s143, c143);
    logic s144 ,c144; 
    fa fa144 (s92, s93, s94, s144, c144);
    logic s145 ,c145; 
    fa fa145 (c92, c93, c94, s145, c145);
    logic s146 ,c146; 
    fa fa146 (s95, s96, s97, s146, c146);
    logic s147 ,c147; 
    fa fa147 (c95, c96, c97, s147, c147);
    logic s148 ,c148; 
    fa fa148 (s98, s99, s100, s148, c148);
    logic s149 ,c149; 
    fa fa149 (s40, c98, c99, s149, c149);
    logic s150 ,c150; 
    fa fa150 (c100, s101, s102, s150, c150);
    logic s151 ,c151; 
    fa fa151 (c101, c102, c103, s151, c151);
    logic s152 ,c152; 
    fa fa152 (s104, s105, s106, s152, c152);
    logic s153 ,c153; 
    fa fa153 (c104, c105, c106, s153, c153);
    logic s154 ,c154; 
    fa fa154 (s107, s108, s109, s154, c154);
    logic s155 ,c155; 
    fa fa155 (c107, c108, c109, s155, c155);
    logic s156 ,c156; 
    fa fa156 (s110, s111, s112, s156, c156);
    logic s157 ,c157; 
    fa fa157 (c110, c111, c112, s157, c157);
    logic s158 ,c158; 
    fa fa158 (s113, s114, s115, s158, c158);
    logic s159 ,c159; 
    fa fa159 (c113, c114, c115, s159, c159);
    logic s160 ,c160; 
    fa fa160 (s116, s117, s118, s160, c160);
    logic s161 ,c161; 
    fa fa161 (c116, c117, c118, s161, c161);
    logic s162 ,c162; 
    fa fa162 (s119, s120, s121, s162, c162);
    logic s163 ,c163; 
    fa fa163 (c119, c120, c121, s163, c163);
    logic s164 ,c164; 
    fa fa164 (s122, s123, s124, s164, c164);
    logic s165 ,c165; 
    fa fa165 (c122, c123, c124, s165, c165);
    logic s166 ,c166; 
    fa fa166 (s125, s126, s127, s166, c166);
    logic s167 ,c167; 
    fa fa167 (s74, c125, c126, s167, c167);
    logic s168 ,c168; 
    fa fa168 (c127, s128, s129, s168, c168);
    logic s169 ,c169; 
    fa fa169 (c128, c129, s130, s169, c169);
    logic s170 ,c170;
    ha ha170 (~IN3[22], c130, s170, c170);
    
    // Wallace Summation Stage 4
    logic s171 ,c171; 
    fa fa171 (s79, c131, s132, s171, c171);
    logic s172 ,c172; 
    fa fa172 (c132, s133, s134, s172, c172);
    logic s173 ,c173; 
    fa fa173 (c133, c134, s135, s173, c173);
    logic s174 ,c174; 
    fa fa174 (c135, c136, s137, s174, c174);
    logic s175 ,c175; 
    fa fa175 (c137, c138, s139, s175, c175);
    logic s176 ,c176; 
    fa fa176 (c139, c140, s141, s176, c176);
    logic s177 ,c177; 
    fa fa177 (c141, c142, s143, s177, c177);
    logic s178 ,c178; 
    fa fa178 (c143, c144, s145, s178, c178);
    logic s179 ,c179; 
    fa fa179 (c145, c146, s147, s179, c179);
    logic s180 ,c180; 
    fa fa180 (s103, c147, c148, s180, c180);
    logic s181 ,c181;
    ha ha181 (s149, s150, s181, c181);
    logic s182 ,c182; 
    fa fa182 (c149, c150, s151, s182, c182);
    logic s183 ,c183; 
    fa fa183 (c151, c152, s153, s183, c183);
    logic s184 ,c184; 
    fa fa184 (c153, c154, s155, s184, c184);
    logic s185 ,c185; 
    fa fa185 (c155, c156, s157, s185, c185);
    logic s186 ,c186; 
    fa fa186 (c157, c158, s159, s186, c186);
    logic s187 ,c187; 
    fa fa187 (c159, c160, s161, s187, c187);
    logic s188 ,c188; 
    fa fa188 (c161, c162, s163, s188, c188);
    logic s189 ,c189; 
    fa fa189 (c163, c164, s165, s189, c189);
    logic s190 ,c190; 
    fa fa190 (c165, c166, s167, s190, c190);
    logic s191 ,c191; 
    fa fa191 (c167, c168, s169, s191, c191);
    logic s192 ,c192;
    ha ha192 (c169, s170, s192, c192);
    
    // Wallace Summation Stage 5
    logic s193 ,c193;
    ha ha193 (c171, s172, s193, c193);
    logic s194 ,c194; 
    fa fa194 (s136, c172, s173, s194, c194);
    logic s195 ,c195; 
    fa fa195 (s138, c173, s174, s195, c195);
    logic s196 ,c196; 
    fa fa196 (s140, c174, s175, s196, c196);
    logic s197 ,c197; 
    fa fa197 (s142, c175, s176, s197, c197);
    logic s198 ,c198; 
    fa fa198 (s144, c176, s177, s198, c198);
    logic s199 ,c199; 
    fa fa199 (s146, c177, s178, s199, c199);
    logic s200 ,c200; 
    fa fa200 (s148, c178, s179, s200, c200);
    logic s201 ,c201; 
    fa fa201 (c179, s180, s181, s201, c201);
    logic s202 ,c202; 
    fa fa202 (s152, c180, c181, s202, c202);
    logic s203 ,c203; 
    fa fa203 (s154, c182, s183, s203, c203);
    logic s204 ,c204; 
    fa fa204 (s156, c183, s184, s204, c204);
    logic s205 ,c205; 
    fa fa205 (s158, c184, s185, s205, c205);
    logic s206 ,c206; 
    fa fa206 (s160, c185, s186, s206, c206);
    logic s207 ,c207; 
    fa fa207 (s162, c186, s187, s207, c207);
    logic s208 ,c208; 
    fa fa208 (s164, c187, s188, s208, c208);
    logic s209 ,c209; 
    fa fa209 (s166, c188, s189, s209, c209);
    logic s210 ,c210; 
    fa fa210 (s168, c189, s190, s210, c210);
    logic s211 ,c211;
    ha ha211 (c190, s191, s211, c211);
    logic s212 ,c212;
    ha ha212 (c191, s192, s212, c212);
    
    // Wallace Summation Stage 6
    logic s213 ,c213;
    ha ha213 (c193, s194, s213, c213);
    logic s214 ,c214;
    ha ha214 (c194, s195, s214, c214);
    logic s215 ,c215;
    ha ha215 (c195, s196, s215, c215);
    logic s216 ,c216;
    ha ha216 (c196, s197, s216, c216);
    logic s217 ,c217;
    ha ha217 (c197, s198, s217, c217);
    logic s218 ,c218;
    ha ha218 (c198, s199, s218, c218);
    logic s219 ,c219;
    ha ha219 (c199, s200, s219, c219);
    logic s220 ,c220;
    ha ha220 (c200, s201, s220, c220);
    logic s221 ,c221; 
    fa fa221 (s182, c201, s202, s221, c221);
    logic s222 ,c222;
    ha ha222 (c202, s203, s222, c222);
    logic s223 ,c223;
    ha ha223 (c203, s204, s223, c223);
    logic s224 ,c224;
    ha ha224 (c204, s205, s224, c224);
    logic s225 ,c225;
    ha ha225 (c205, s206, s225, c225);
    logic s226 ,c226;
    ha ha226 (c206, s207, s226, c226);
    logic s227 ,c227;
    ha ha227 (c207, s208, s227, c227);
    logic s228 ,c228;
    ha ha228 (c208, s209, s228, c228);
    logic s229 ,c229;
    ha ha229 (c209, s210, s229, c229);
    logic s230 ,c230;
    ha ha230 (c210, s211, s230, c230);
    logic s231 ,c231;
    ha ha231 (c211, s212, s231, c231);
    
    assign result[3] = s193;
    assign result[4] = s213;
    logic [18:0] adder_result;
    JSkCond_18 final_adder ({c230, c229, c228, c227, c226, c225, c224, c223, c222, c221, c220, c219, c218, c217, c216, c215, c214, c213 }, {s231, s230, s229, s228, s227, s226, s225, s224, s223, s222, s221, s220, s219, s218, s217, s216, s215, s214 }, adder_result );
    assign result[22:5] = adder_result[17:0];
    

endmodule

module JSkCond_18 ( 
        input logic [17:0] IN1,
        input logic [17:0] IN2,
        output logic [18:0] OUT);
    
    wire logic [17:0] p_0;
    wire logic [17:0] g_0;
    assign g_0 = IN1 & IN2;
    assign p_0 = IN1 ^ IN2;
    
    // J. Sklansky – Conditional Adder 

    
    // Stage 1 - prop from 1 to 1 per group.
    wire logic p_1_1;
    wire logic g_1_1;
    assign p_1_1 = p_0[1] & p_0[0];
    assign g_1_1 = (p_0[1] & g_0[0]) | g_0[1];
    wire logic p_1_3;
    wire logic g_1_3;
    assign p_1_3 = p_0[3] & p_0[2];
    assign g_1_3 = (p_0[3] & g_0[2]) | g_0[3];
    wire logic p_1_5;
    wire logic g_1_5;
    assign p_1_5 = p_0[5] & p_0[4];
    assign g_1_5 = (p_0[5] & g_0[4]) | g_0[5];
    wire logic p_1_7;
    wire logic g_1_7;
    assign p_1_7 = p_0[7] & p_0[6];
    assign g_1_7 = (p_0[7] & g_0[6]) | g_0[7];
    wire logic p_1_9;
    wire logic g_1_9;
    assign p_1_9 = p_0[9] & p_0[8];
    assign g_1_9 = (p_0[9] & g_0[8]) | g_0[9];
    wire logic p_1_11;
    wire logic g_1_11;
    assign p_1_11 = p_0[11] & p_0[10];
    assign g_1_11 = (p_0[11] & g_0[10]) | g_0[11];
    wire logic p_1_13;
    wire logic g_1_13;
    assign p_1_13 = p_0[13] & p_0[12];
    assign g_1_13 = (p_0[13] & g_0[12]) | g_0[13];
    wire logic p_1_15;
    wire logic g_1_15;
    assign p_1_15 = p_0[15] & p_0[14];
    assign g_1_15 = (p_0[15] & g_0[14]) | g_0[15];
    wire logic p_1_17;
    wire logic g_1_17;
    assign p_1_17 = p_0[17] & p_0[16];
    assign g_1_17 = (p_0[17] & g_0[16]) | g_0[17];
    
    // Stage 2 - prop from 1 to 2 per group.
    wire logic p_2_2;
    wire logic g_2_2;
    assign p_2_2 = p_0[2] & p_1_1;
    assign g_2_2 = (p_0[2] & g_1_1) | g_0[2];
    wire logic p_2_3;
    wire logic g_2_3;
    assign p_2_3 = p_1_3 & p_1_1;
    assign g_2_3 = (p_1_3 & g_1_1) | g_1_3;
    wire logic p_2_6;
    wire logic g_2_6;
    assign p_2_6 = p_0[6] & p_1_5;
    assign g_2_6 = (p_0[6] & g_1_5) | g_0[6];
    wire logic p_2_7;
    wire logic g_2_7;
    assign p_2_7 = p_1_7 & p_1_5;
    assign g_2_7 = (p_1_7 & g_1_5) | g_1_7;
    wire logic p_2_10;
    wire logic g_2_10;
    assign p_2_10 = p_0[10] & p_1_9;
    assign g_2_10 = (p_0[10] & g_1_9) | g_0[10];
    wire logic p_2_11;
    wire logic g_2_11;
    assign p_2_11 = p_1_11 & p_1_9;
    assign g_2_11 = (p_1_11 & g_1_9) | g_1_11;
    wire logic p_2_14;
    wire logic g_2_14;
    assign p_2_14 = p_0[14] & p_1_13;
    assign g_2_14 = (p_0[14] & g_1_13) | g_0[14];
    wire logic p_2_15;
    wire logic g_2_15;
    assign p_2_15 = p_1_15 & p_1_13;
    assign g_2_15 = (p_1_15 & g_1_13) | g_1_15;
    
    // Stage 3 - prop from 1 to 4 per group.
    wire logic p_3_4;
    wire logic g_3_4;
    assign p_3_4 = p_0[4] & p_2_3;
    assign g_3_4 = (p_0[4] & g_2_3) | g_0[4];
    wire logic p_3_5;
    wire logic g_3_5;
    assign p_3_5 = p_1_5 & p_2_3;
    assign g_3_5 = (p_1_5 & g_2_3) | g_1_5;
    wire logic p_3_6;
    wire logic g_3_6;
    assign p_3_6 = p_2_6 & p_2_3;
    assign g_3_6 = (p_2_6 & g_2_3) | g_2_6;
    wire logic p_3_7;
    wire logic g_3_7;
    assign p_3_7 = p_2_7 & p_2_3;
    assign g_3_7 = (p_2_7 & g_2_3) | g_2_7;
    wire logic p_3_12;
    wire logic g_3_12;
    assign p_3_12 = p_0[12] & p_2_11;
    assign g_3_12 = (p_0[12] & g_2_11) | g_0[12];
    wire logic p_3_13;
    wire logic g_3_13;
    assign p_3_13 = p_1_13 & p_2_11;
    assign g_3_13 = (p_1_13 & g_2_11) | g_1_13;
    wire logic p_3_14;
    wire logic g_3_14;
    assign p_3_14 = p_2_14 & p_2_11;
    assign g_3_14 = (p_2_14 & g_2_11) | g_2_14;
    wire logic p_3_15;
    wire logic g_3_15;
    assign p_3_15 = p_2_15 & p_2_11;
    assign g_3_15 = (p_2_15 & g_2_11) | g_2_15;
    
    // Stage 4 - prop from 1 to 8 per group.
    wire logic p_4_8;
    wire logic g_4_8;
    assign p_4_8 = p_0[8] & p_3_7;
    assign g_4_8 = (p_0[8] & g_3_7) | g_0[8];
    wire logic p_4_9;
    wire logic g_4_9;
    assign p_4_9 = p_1_9 & p_3_7;
    assign g_4_9 = (p_1_9 & g_3_7) | g_1_9;
    wire logic p_4_10;
    wire logic g_4_10;
    assign p_4_10 = p_2_10 & p_3_7;
    assign g_4_10 = (p_2_10 & g_3_7) | g_2_10;
    wire logic p_4_11;
    wire logic g_4_11;
    assign p_4_11 = p_2_11 & p_3_7;
    assign g_4_11 = (p_2_11 & g_3_7) | g_2_11;
    wire logic p_4_12;
    wire logic g_4_12;
    assign p_4_12 = p_3_12 & p_3_7;
    assign g_4_12 = (p_3_12 & g_3_7) | g_3_12;
    wire logic p_4_13;
    wire logic g_4_13;
    assign p_4_13 = p_3_13 & p_3_7;
    assign g_4_13 = (p_3_13 & g_3_7) | g_3_13;
    wire logic p_4_14;
    wire logic g_4_14;
    assign p_4_14 = p_3_14 & p_3_7;
    assign g_4_14 = (p_3_14 & g_3_7) | g_3_14;
    wire logic p_4_15;
    wire logic g_4_15;
    assign p_4_15 = p_3_15 & p_3_7;
    assign g_4_15 = (p_3_15 & g_3_7) | g_3_15;
    
    // Stage 5 - prop from 1 to 16 per group.
    wire logic p_5_16;
    wire logic g_5_16;
    assign p_5_16 = p_0[16] & p_4_15;
    assign g_5_16 = (p_0[16] & g_4_15) | g_0[16];
    wire logic p_5_17;
    wire logic g_5_17;
    assign p_5_17 = p_1_17 & p_4_15;
    assign g_5_17 = (p_1_17 & g_4_15) | g_1_17;
    
    // JSkCondA postprocess 
    assign OUT[0] = p_0[0];
    assign OUT[1] = p_0[1] ^ g_0[0];
    assign OUT[2] = p_0[2] ^ g_1_1;
    assign OUT[3] = p_0[3] ^ g_2_2;
    assign OUT[4] = p_0[4] ^ g_2_3;
    assign OUT[5] = p_0[5] ^ g_3_4;
    assign OUT[6] = p_0[6] ^ g_3_5;
    assign OUT[7] = p_0[7] ^ g_3_6;
    assign OUT[8] = p_0[8] ^ g_3_7;
    assign OUT[9] = p_0[9] ^ g_4_8;
    assign OUT[10] = p_0[10] ^ g_4_9;
    assign OUT[11] = p_0[11] ^ g_4_10;
    assign OUT[12] = p_0[12] ^ g_4_11;
    assign OUT[13] = p_0[13] ^ g_4_12;
    assign OUT[14] = p_0[14] ^ g_4_13;
    assign OUT[15] = p_0[15] ^ g_4_14;
    assign OUT[16] = p_0[16] ^ g_4_15;
    assign OUT[17] = p_0[17] ^ g_5_16;
    assign OUT[18] = g_5_17;
endmodule

module JSkCond_18_spec (
        input logic [17:0] IN1,
        input logic [17:0] IN2,
        output logic adder_correct,
        output logic [18:0] spec_res);
    
    assign spec_res = IN1 + IN2;
    wire [18:0] adder_res;
    JSkCond_18 adder(IN1, IN2, adder_res);
    assign adder_correct = ((spec_res == adder_res) ? 1 : 0);
    
endmodule



module ha (
        input logic a,
        input logic b,
        output logic s,
        output logic c);
    
    assign s = a ^ b;
    assign c = a & b;
endmodule



module fa (
        input logic x,
        input logic y,
        input logic z,
        output logic s,
        output logic c);
    
    assign s = x ^ y ^ z;
    assign c = (x & y) | (x & z) | (y & z);
endmodule

module Four2Two 
        #(parameter WIDTH=1) (
        input logic [WIDTH-1:0] in1,
        input logic [WIDTH-1:0] in2,
        input logic [WIDTH-1:0] in3,
        input logic [WIDTH-1:0] in4,
        input logic cin,
        output logic [WIDTH-1:0] sum,
        output logic [WIDTH-1:0] carry,
        output logic cout);
    
    wire logic [WIDTH:0] temp1;
    assign temp1 = {((in1 ^ in2)&in3 | in1 & ~(in1^in2)),cin};
    assign sum = ((in1 ^ in2) ^ in3 ^ in4) ^ temp1[WIDTH-1:0];
    assign carry = ((in1 ^ in2) ^ in3 ^ in4) & temp1[WIDTH-1:0] | in4 & ~((in1 ^ in2) ^ in3 ^ in4);
    assign cout = temp1[WIDTH];
endmodule




