// Note: The license below is based on the template at:
// http://opensource.org/licenses/BSD-3-Clause
// Copyright (C) 2020 Regents of the University of Texas
//All rights reserved.

// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are
// met:

// o Redistributions of source code must retain the above copyright
//   notice, this list of conditions and the following disclaimer.

// o Redistributions in binary form must reproduce the above copyright
//   notice, this list of conditions and the following disclaimer in the
//   documentation and/or other materials provided with the distribution.

// o Neither the name of the copyright holders nor the names of its
//   contributors may be used to endorse or promote products derived
//   from this software without specific prior written permission.

// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

// Original Author(s):
// Mertcan Temel         <mert@utexas.edu>

// DO NOT REMOVE:
// This file is generated by Temel's multiplier generator. Download from https://github.com/temelmertcan/multgen.


// Specification module to help understand what the design implements.
module MAC_DT_SB8_HC_32x32_plus_32_63to0_spec (
        input logic [31:0] IN1,
        input logic [31:0] IN2,
        input logic [31:0] IN3,
        output logic design_is_correct, // is set to 1 iff the output of MAC_DT_SB8_HC_32x32_plus_32_63to0  matches its spec.
        output logic [63:0] design_res,
        output logic [63:0] spec_res);
    
    assign spec_res = signed'(IN1) * signed'(IN2) + signed'(IN3);
    MAC_DT_SB8_HC_32x32_plus_32_63to0 mult(IN1, IN2, IN3, design_res);
    assign design_is_correct = ((spec_res == design_res) ? 1 : 0);
    
endmodule

module DT_SB8_32x32(
        input logic [31:0] IN1,
        input logic [31:0] IN2,
        output logic [63:0] result0,
        output logic [63:0] result1);
    
    
// Creating Partial Products 

    wire logic const1;
    assign const1 = 1'b1;
    wire [31:0] mult = IN1;
    wire [31:0] mcand = IN2;
    wire [33:0] mcand_1x;
    wire [33:0] mcand_2x;
    wire [33:0] mcand_3x;
    wire [33:0] mcand_4x;
    assign mcand_1x = {{2{mcand[31]}},  mcand};
    assign mcand_2x = {{1{mcand[31]}},  mcand, 1'b0};
    HC_34_carry calc_mcand_3x (1'b0, mcand_1x, mcand_2x, mcand_3x);
    assign mcand_4x = {{0{mcand[31]}},  mcand, 2'b0};
    
    // Booth Radix-8 Partial Products. Multiplier selectors: mult[2] mult[1] mult[0] 1'b0
    wire logic select_0_0, select_e_0, select_2x_0, select_3x_0, select_4x_0, tcomp0, select_ne_0, select_n2x_0, select_n3x_0, select_n4x_0;
    assign select_0_0 =  &{mult[2], mult[1], mult[0], 1'b0} | ~|{mult[2], mult[1], mult[0], 1'b0};
    assign select_e_0 = ((~ mult[2]) & (~ mult[1]) & (mult[0] ^ 1'b0));
    assign select_ne_0 = mult[2] & mult[1] &  (mult[0] ^ 1'b0);
    assign select_2x_0 = (~ mult[2]) & (mult[1] ^ mult[0])& (mult[1] ^ 1'b0);
    assign select_n2x_0 = mult[2] & (mult[1] ^ mult[0])& (mult[1] ^ 1'b0);
    assign select_3x_0 = ((~ mult[2]) & mult[1] & (mult[0] ^ 1'b0));
    assign select_n3x_0 = (mult[2] & ~mult[1] & (mult[0] ^ 1'b0));
    assign select_4x_0 = ~mult[2] & mult[1] & mult[0] & 1'b0;
    assign select_n4x_0 = mult[2] & ~mult[1] & ~mult[0] & ~1'b0;
    reg [33:0] pp_0;
    always @(*) begin
       case (1'b1)
          select_0_0   : pp_0 = 0; 
          select_e_0   : pp_0 = mcand_1x; 
          select_2x_0  : pp_0 = mcand_2x; 
          select_3x_0  : pp_0 = mcand_3x; 
          select_4x_0  : pp_0 = mcand_4x; 
          select_n4x_0 : pp_0 = (~ mcand_4x); 
          select_n3x_0 : pp_0 = (~ mcand_3x); 
          select_n2x_0 : pp_0 = (~ mcand_2x); 
          select_ne_0  : pp_0 = (~ mcand_1x); 
       endcase 
       pp_0[33] = ~pp_0[33]; // flip the MSB 
    end
    assign tcomp0 = select_ne_0 | select_n4x_0 | select_n3x_0 | select_n2x_0;
    
    // Booth Radix-8 Partial Products. Multiplier selectors: mult[5] mult[4] mult[3] mult[2]
    wire logic select_0_1, select_e_1, select_2x_1, select_3x_1, select_4x_1, tcomp1, select_ne_1, select_n2x_1, select_n3x_1, select_n4x_1;
    assign select_0_1 =  &{mult[5], mult[4], mult[3], mult[2]} | ~|{mult[5], mult[4], mult[3], mult[2]};
    assign select_e_1 = ((~ mult[5]) & (~ mult[4]) & (mult[3] ^ mult[2]));
    assign select_ne_1 = mult[5] & mult[4] &  (mult[3] ^ mult[2]);
    assign select_2x_1 = (~ mult[5]) & (mult[4] ^ mult[3])& (mult[4] ^ mult[2]);
    assign select_n2x_1 = mult[5] & (mult[4] ^ mult[3])& (mult[4] ^ mult[2]);
    assign select_3x_1 = ((~ mult[5]) & mult[4] & (mult[3] ^ mult[2]));
    assign select_n3x_1 = (mult[5] & ~mult[4] & (mult[3] ^ mult[2]));
    assign select_4x_1 = ~mult[5] & mult[4] & mult[3] & mult[2];
    assign select_n4x_1 = mult[5] & ~mult[4] & ~mult[3] & ~mult[2];
    reg [33:0] pp_1;
    always @(*) begin
       case (1'b1)
          select_0_1   : pp_1 = 0; 
          select_e_1   : pp_1 = mcand_1x; 
          select_2x_1  : pp_1 = mcand_2x; 
          select_3x_1  : pp_1 = mcand_3x; 
          select_4x_1  : pp_1 = mcand_4x; 
          select_n4x_1 : pp_1 = (~ mcand_4x); 
          select_n3x_1 : pp_1 = (~ mcand_3x); 
          select_n2x_1 : pp_1 = (~ mcand_2x); 
          select_ne_1  : pp_1 = (~ mcand_1x); 
       endcase 
       pp_1[33] = ~pp_1[33]; // flip the MSB 
    end
    assign tcomp1 = select_ne_1 | select_n4x_1 | select_n3x_1 | select_n2x_1;
    
    // Booth Radix-8 Partial Products. Multiplier selectors: mult[8] mult[7] mult[6] mult[5]
    wire logic select_0_2, select_e_2, select_2x_2, select_3x_2, select_4x_2, tcomp2, select_ne_2, select_n2x_2, select_n3x_2, select_n4x_2;
    assign select_0_2 =  &{mult[8], mult[7], mult[6], mult[5]} | ~|{mult[8], mult[7], mult[6], mult[5]};
    assign select_e_2 = ((~ mult[8]) & (~ mult[7]) & (mult[6] ^ mult[5]));
    assign select_ne_2 = mult[8] & mult[7] &  (mult[6] ^ mult[5]);
    assign select_2x_2 = (~ mult[8]) & (mult[7] ^ mult[6])& (mult[7] ^ mult[5]);
    assign select_n2x_2 = mult[8] & (mult[7] ^ mult[6])& (mult[7] ^ mult[5]);
    assign select_3x_2 = ((~ mult[8]) & mult[7] & (mult[6] ^ mult[5]));
    assign select_n3x_2 = (mult[8] & ~mult[7] & (mult[6] ^ mult[5]));
    assign select_4x_2 = ~mult[8] & mult[7] & mult[6] & mult[5];
    assign select_n4x_2 = mult[8] & ~mult[7] & ~mult[6] & ~mult[5];
    reg [33:0] pp_2;
    always @(*) begin
       case (1'b1)
          select_0_2   : pp_2 = 0; 
          select_e_2   : pp_2 = mcand_1x; 
          select_2x_2  : pp_2 = mcand_2x; 
          select_3x_2  : pp_2 = mcand_3x; 
          select_4x_2  : pp_2 = mcand_4x; 
          select_n4x_2 : pp_2 = (~ mcand_4x); 
          select_n3x_2 : pp_2 = (~ mcand_3x); 
          select_n2x_2 : pp_2 = (~ mcand_2x); 
          select_ne_2  : pp_2 = (~ mcand_1x); 
       endcase 
       pp_2[33] = ~pp_2[33]; // flip the MSB 
    end
    assign tcomp2 = select_ne_2 | select_n4x_2 | select_n3x_2 | select_n2x_2;
    
    // Booth Radix-8 Partial Products. Multiplier selectors: mult[11] mult[10] mult[9] mult[8]
    wire logic select_0_3, select_e_3, select_2x_3, select_3x_3, select_4x_3, tcomp3, select_ne_3, select_n2x_3, select_n3x_3, select_n4x_3;
    assign select_0_3 =  &{mult[11], mult[10], mult[9], mult[8]} | ~|{mult[11], mult[10], mult[9], mult[8]};
    assign select_e_3 = ((~ mult[11]) & (~ mult[10]) & (mult[9] ^ mult[8]));
    assign select_ne_3 = mult[11] & mult[10] &  (mult[9] ^ mult[8]);
    assign select_2x_3 = (~ mult[11]) & (mult[10] ^ mult[9])& (mult[10] ^ mult[8]);
    assign select_n2x_3 = mult[11] & (mult[10] ^ mult[9])& (mult[10] ^ mult[8]);
    assign select_3x_3 = ((~ mult[11]) & mult[10] & (mult[9] ^ mult[8]));
    assign select_n3x_3 = (mult[11] & ~mult[10] & (mult[9] ^ mult[8]));
    assign select_4x_3 = ~mult[11] & mult[10] & mult[9] & mult[8];
    assign select_n4x_3 = mult[11] & ~mult[10] & ~mult[9] & ~mult[8];
    reg [33:0] pp_3;
    always @(*) begin
       case (1'b1)
          select_0_3   : pp_3 = 0; 
          select_e_3   : pp_3 = mcand_1x; 
          select_2x_3  : pp_3 = mcand_2x; 
          select_3x_3  : pp_3 = mcand_3x; 
          select_4x_3  : pp_3 = mcand_4x; 
          select_n4x_3 : pp_3 = (~ mcand_4x); 
          select_n3x_3 : pp_3 = (~ mcand_3x); 
          select_n2x_3 : pp_3 = (~ mcand_2x); 
          select_ne_3  : pp_3 = (~ mcand_1x); 
       endcase 
       pp_3[33] = ~pp_3[33]; // flip the MSB 
    end
    assign tcomp3 = select_ne_3 | select_n4x_3 | select_n3x_3 | select_n2x_3;
    
    // Booth Radix-8 Partial Products. Multiplier selectors: mult[14] mult[13] mult[12] mult[11]
    wire logic select_0_4, select_e_4, select_2x_4, select_3x_4, select_4x_4, tcomp4, select_ne_4, select_n2x_4, select_n3x_4, select_n4x_4;
    assign select_0_4 =  &{mult[14], mult[13], mult[12], mult[11]} | ~|{mult[14], mult[13], mult[12], mult[11]};
    assign select_e_4 = ((~ mult[14]) & (~ mult[13]) & (mult[12] ^ mult[11]));
    assign select_ne_4 = mult[14] & mult[13] &  (mult[12] ^ mult[11]);
    assign select_2x_4 = (~ mult[14]) & (mult[13] ^ mult[12])& (mult[13] ^ mult[11]);
    assign select_n2x_4 = mult[14] & (mult[13] ^ mult[12])& (mult[13] ^ mult[11]);
    assign select_3x_4 = ((~ mult[14]) & mult[13] & (mult[12] ^ mult[11]));
    assign select_n3x_4 = (mult[14] & ~mult[13] & (mult[12] ^ mult[11]));
    assign select_4x_4 = ~mult[14] & mult[13] & mult[12] & mult[11];
    assign select_n4x_4 = mult[14] & ~mult[13] & ~mult[12] & ~mult[11];
    reg [33:0] pp_4;
    always @(*) begin
       case (1'b1)
          select_0_4   : pp_4 = 0; 
          select_e_4   : pp_4 = mcand_1x; 
          select_2x_4  : pp_4 = mcand_2x; 
          select_3x_4  : pp_4 = mcand_3x; 
          select_4x_4  : pp_4 = mcand_4x; 
          select_n4x_4 : pp_4 = (~ mcand_4x); 
          select_n3x_4 : pp_4 = (~ mcand_3x); 
          select_n2x_4 : pp_4 = (~ mcand_2x); 
          select_ne_4  : pp_4 = (~ mcand_1x); 
       endcase 
       pp_4[33] = ~pp_4[33]; // flip the MSB 
    end
    assign tcomp4 = select_ne_4 | select_n4x_4 | select_n3x_4 | select_n2x_4;
    
    // Booth Radix-8 Partial Products. Multiplier selectors: mult[17] mult[16] mult[15] mult[14]
    wire logic select_0_5, select_e_5, select_2x_5, select_3x_5, select_4x_5, tcomp5, select_ne_5, select_n2x_5, select_n3x_5, select_n4x_5;
    assign select_0_5 =  &{mult[17], mult[16], mult[15], mult[14]} | ~|{mult[17], mult[16], mult[15], mult[14]};
    assign select_e_5 = ((~ mult[17]) & (~ mult[16]) & (mult[15] ^ mult[14]));
    assign select_ne_5 = mult[17] & mult[16] &  (mult[15] ^ mult[14]);
    assign select_2x_5 = (~ mult[17]) & (mult[16] ^ mult[15])& (mult[16] ^ mult[14]);
    assign select_n2x_5 = mult[17] & (mult[16] ^ mult[15])& (mult[16] ^ mult[14]);
    assign select_3x_5 = ((~ mult[17]) & mult[16] & (mult[15] ^ mult[14]));
    assign select_n3x_5 = (mult[17] & ~mult[16] & (mult[15] ^ mult[14]));
    assign select_4x_5 = ~mult[17] & mult[16] & mult[15] & mult[14];
    assign select_n4x_5 = mult[17] & ~mult[16] & ~mult[15] & ~mult[14];
    reg [33:0] pp_5;
    always @(*) begin
       case (1'b1)
          select_0_5   : pp_5 = 0; 
          select_e_5   : pp_5 = mcand_1x; 
          select_2x_5  : pp_5 = mcand_2x; 
          select_3x_5  : pp_5 = mcand_3x; 
          select_4x_5  : pp_5 = mcand_4x; 
          select_n4x_5 : pp_5 = (~ mcand_4x); 
          select_n3x_5 : pp_5 = (~ mcand_3x); 
          select_n2x_5 : pp_5 = (~ mcand_2x); 
          select_ne_5  : pp_5 = (~ mcand_1x); 
       endcase 
       pp_5[33] = ~pp_5[33]; // flip the MSB 
    end
    assign tcomp5 = select_ne_5 | select_n4x_5 | select_n3x_5 | select_n2x_5;
    
    // Booth Radix-8 Partial Products. Multiplier selectors: mult[20] mult[19] mult[18] mult[17]
    wire logic select_0_6, select_e_6, select_2x_6, select_3x_6, select_4x_6, tcomp6, select_ne_6, select_n2x_6, select_n3x_6, select_n4x_6;
    assign select_0_6 =  &{mult[20], mult[19], mult[18], mult[17]} | ~|{mult[20], mult[19], mult[18], mult[17]};
    assign select_e_6 = ((~ mult[20]) & (~ mult[19]) & (mult[18] ^ mult[17]));
    assign select_ne_6 = mult[20] & mult[19] &  (mult[18] ^ mult[17]);
    assign select_2x_6 = (~ mult[20]) & (mult[19] ^ mult[18])& (mult[19] ^ mult[17]);
    assign select_n2x_6 = mult[20] & (mult[19] ^ mult[18])& (mult[19] ^ mult[17]);
    assign select_3x_6 = ((~ mult[20]) & mult[19] & (mult[18] ^ mult[17]));
    assign select_n3x_6 = (mult[20] & ~mult[19] & (mult[18] ^ mult[17]));
    assign select_4x_6 = ~mult[20] & mult[19] & mult[18] & mult[17];
    assign select_n4x_6 = mult[20] & ~mult[19] & ~mult[18] & ~mult[17];
    reg [33:0] pp_6;
    always @(*) begin
       case (1'b1)
          select_0_6   : pp_6 = 0; 
          select_e_6   : pp_6 = mcand_1x; 
          select_2x_6  : pp_6 = mcand_2x; 
          select_3x_6  : pp_6 = mcand_3x; 
          select_4x_6  : pp_6 = mcand_4x; 
          select_n4x_6 : pp_6 = (~ mcand_4x); 
          select_n3x_6 : pp_6 = (~ mcand_3x); 
          select_n2x_6 : pp_6 = (~ mcand_2x); 
          select_ne_6  : pp_6 = (~ mcand_1x); 
       endcase 
       pp_6[33] = ~pp_6[33]; // flip the MSB 
    end
    assign tcomp6 = select_ne_6 | select_n4x_6 | select_n3x_6 | select_n2x_6;
    
    // Booth Radix-8 Partial Products. Multiplier selectors: mult[23] mult[22] mult[21] mult[20]
    wire logic select_0_7, select_e_7, select_2x_7, select_3x_7, select_4x_7, tcomp7, select_ne_7, select_n2x_7, select_n3x_7, select_n4x_7;
    assign select_0_7 =  &{mult[23], mult[22], mult[21], mult[20]} | ~|{mult[23], mult[22], mult[21], mult[20]};
    assign select_e_7 = ((~ mult[23]) & (~ mult[22]) & (mult[21] ^ mult[20]));
    assign select_ne_7 = mult[23] & mult[22] &  (mult[21] ^ mult[20]);
    assign select_2x_7 = (~ mult[23]) & (mult[22] ^ mult[21])& (mult[22] ^ mult[20]);
    assign select_n2x_7 = mult[23] & (mult[22] ^ mult[21])& (mult[22] ^ mult[20]);
    assign select_3x_7 = ((~ mult[23]) & mult[22] & (mult[21] ^ mult[20]));
    assign select_n3x_7 = (mult[23] & ~mult[22] & (mult[21] ^ mult[20]));
    assign select_4x_7 = ~mult[23] & mult[22] & mult[21] & mult[20];
    assign select_n4x_7 = mult[23] & ~mult[22] & ~mult[21] & ~mult[20];
    reg [33:0] pp_7;
    always @(*) begin
       case (1'b1)
          select_0_7   : pp_7 = 0; 
          select_e_7   : pp_7 = mcand_1x; 
          select_2x_7  : pp_7 = mcand_2x; 
          select_3x_7  : pp_7 = mcand_3x; 
          select_4x_7  : pp_7 = mcand_4x; 
          select_n4x_7 : pp_7 = (~ mcand_4x); 
          select_n3x_7 : pp_7 = (~ mcand_3x); 
          select_n2x_7 : pp_7 = (~ mcand_2x); 
          select_ne_7  : pp_7 = (~ mcand_1x); 
       endcase 
       pp_7[33] = ~pp_7[33]; // flip the MSB 
    end
    assign tcomp7 = select_ne_7 | select_n4x_7 | select_n3x_7 | select_n2x_7;
    
    // Booth Radix-8 Partial Products. Multiplier selectors: mult[26] mult[25] mult[24] mult[23]
    wire logic select_0_8, select_e_8, select_2x_8, select_3x_8, select_4x_8, tcomp8, select_ne_8, select_n2x_8, select_n3x_8, select_n4x_8;
    assign select_0_8 =  &{mult[26], mult[25], mult[24], mult[23]} | ~|{mult[26], mult[25], mult[24], mult[23]};
    assign select_e_8 = ((~ mult[26]) & (~ mult[25]) & (mult[24] ^ mult[23]));
    assign select_ne_8 = mult[26] & mult[25] &  (mult[24] ^ mult[23]);
    assign select_2x_8 = (~ mult[26]) & (mult[25] ^ mult[24])& (mult[25] ^ mult[23]);
    assign select_n2x_8 = mult[26] & (mult[25] ^ mult[24])& (mult[25] ^ mult[23]);
    assign select_3x_8 = ((~ mult[26]) & mult[25] & (mult[24] ^ mult[23]));
    assign select_n3x_8 = (mult[26] & ~mult[25] & (mult[24] ^ mult[23]));
    assign select_4x_8 = ~mult[26] & mult[25] & mult[24] & mult[23];
    assign select_n4x_8 = mult[26] & ~mult[25] & ~mult[24] & ~mult[23];
    reg [33:0] pp_8;
    always @(*) begin
       case (1'b1)
          select_0_8   : pp_8 = 0; 
          select_e_8   : pp_8 = mcand_1x; 
          select_2x_8  : pp_8 = mcand_2x; 
          select_3x_8  : pp_8 = mcand_3x; 
          select_4x_8  : pp_8 = mcand_4x; 
          select_n4x_8 : pp_8 = (~ mcand_4x); 
          select_n3x_8 : pp_8 = (~ mcand_3x); 
          select_n2x_8 : pp_8 = (~ mcand_2x); 
          select_ne_8  : pp_8 = (~ mcand_1x); 
       endcase 
       pp_8[33] = ~pp_8[33]; // flip the MSB 
    end
    assign tcomp8 = select_ne_8 | select_n4x_8 | select_n3x_8 | select_n2x_8;
    
    // Booth Radix-8 Partial Products. Multiplier selectors: mult[29] mult[28] mult[27] mult[26]
    wire logic select_0_9, select_e_9, select_2x_9, select_3x_9, select_4x_9, tcomp9, select_ne_9, select_n2x_9, select_n3x_9, select_n4x_9;
    assign select_0_9 =  &{mult[29], mult[28], mult[27], mult[26]} | ~|{mult[29], mult[28], mult[27], mult[26]};
    assign select_e_9 = ((~ mult[29]) & (~ mult[28]) & (mult[27] ^ mult[26]));
    assign select_ne_9 = mult[29] & mult[28] &  (mult[27] ^ mult[26]);
    assign select_2x_9 = (~ mult[29]) & (mult[28] ^ mult[27])& (mult[28] ^ mult[26]);
    assign select_n2x_9 = mult[29] & (mult[28] ^ mult[27])& (mult[28] ^ mult[26]);
    assign select_3x_9 = ((~ mult[29]) & mult[28] & (mult[27] ^ mult[26]));
    assign select_n3x_9 = (mult[29] & ~mult[28] & (mult[27] ^ mult[26]));
    assign select_4x_9 = ~mult[29] & mult[28] & mult[27] & mult[26];
    assign select_n4x_9 = mult[29] & ~mult[28] & ~mult[27] & ~mult[26];
    reg [33:0] pp_9;
    always @(*) begin
       case (1'b1)
          select_0_9   : pp_9 = 0; 
          select_e_9   : pp_9 = mcand_1x; 
          select_2x_9  : pp_9 = mcand_2x; 
          select_3x_9  : pp_9 = mcand_3x; 
          select_4x_9  : pp_9 = mcand_4x; 
          select_n4x_9 : pp_9 = (~ mcand_4x); 
          select_n3x_9 : pp_9 = (~ mcand_3x); 
          select_n2x_9 : pp_9 = (~ mcand_2x); 
          select_ne_9  : pp_9 = (~ mcand_1x); 
       endcase 
       pp_9[33] = ~pp_9[33]; // flip the MSB 
    end
    assign tcomp9 = select_ne_9 | select_n4x_9 | select_n3x_9 | select_n2x_9;
    
    // Booth Radix-8 Partial Products. Multiplier selectors: mult[31] mult[31] mult[30] mult[29]
    wire logic select_0_10, select_e_10, select_2x_10, select_3x_10, select_4x_10, tcomp10, select_ne_10, select_n2x_10, select_n3x_10, select_n4x_10;
    assign select_0_10 =  &{mult[31], mult[31], mult[30], mult[29]} | ~|{mult[31], mult[31], mult[30], mult[29]};
    assign select_e_10 = ((~ mult[31]) & (~ mult[31]) & (mult[30] ^ mult[29]));
    assign select_ne_10 = mult[31] & mult[31] &  (mult[30] ^ mult[29]);
    assign select_2x_10 = (~ mult[31]) & (mult[31] ^ mult[30])& (mult[31] ^ mult[29]);
    assign select_n2x_10 = mult[31] & (mult[31] ^ mult[30])& (mult[31] ^ mult[29]);
    assign select_3x_10 = ((~ mult[31]) & mult[31] & (mult[30] ^ mult[29]));
    assign select_n3x_10 = (mult[31] & ~mult[31] & (mult[30] ^ mult[29]));
    assign select_4x_10 = ~mult[31] & mult[31] & mult[30] & mult[29];
    assign select_n4x_10 = mult[31] & ~mult[31] & ~mult[30] & ~mult[29];
    reg [33:0] pp_10;
    always @(*) begin
       case (1'b1)
          select_0_10   : pp_10 = 0; 
          select_e_10   : pp_10 = mcand_1x; 
          select_2x_10  : pp_10 = mcand_2x; 
          select_3x_10  : pp_10 = mcand_3x; 
          select_4x_10  : pp_10 = mcand_4x; 
          select_n4x_10 : pp_10 = (~ mcand_4x); 
          select_n3x_10 : pp_10 = (~ mcand_3x); 
          select_n2x_10 : pp_10 = (~ mcand_2x); 
          select_ne_10  : pp_10 = (~ mcand_1x); 
       endcase 
       pp_10[33] = ~pp_10[33]; // flip the MSB 
    end
    assign tcomp10 = select_ne_10 | select_n4x_10 | select_n3x_10 | select_n2x_10;
    
    // The values to be summed in the summation tree, from LSB (left) to MSB:
     // pp_0[0]   pp_0[1]   pp_0[2]   pp_0[3]   pp_0[4]   pp_0[5]   pp_0[6]   pp_0[7]   pp_0[8]   pp_0[9]   pp_0[10]  pp_0[11]  pp_0[12]  pp_0[13]  pp_0[14]  pp_0[15]  pp_0[16]  pp_0[17]  pp_0[18]  pp_0[19]  pp_0[20]  pp_0[21]  pp_0[22]  pp_0[23]  pp_0[24]  pp_0[25]  pp_0[26]  pp_0[27]  pp_0[28]  pp_0[29]  pp_0[30]  pp_0[31]  pp_0[32]  pp_0[33]    --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --      
     //   --        --        --      pp_1[0]   pp_1[1]   pp_1[2]   pp_1[3]   pp_1[4]   pp_1[5]   pp_1[6]   pp_1[7]   pp_1[8]   pp_1[9]   pp_1[10]  pp_1[11]  pp_1[12]  pp_1[13]  pp_1[14]  pp_1[15]  pp_1[16]  pp_1[17]  pp_1[18]  pp_1[19]  pp_1[20]  pp_1[21]  pp_1[22]  pp_1[23]  pp_1[24]  pp_1[25]  pp_1[26]  pp_1[27]  pp_1[28]  pp_1[29]  pp_1[30]  pp_1[31]  pp_1[32]  pp_1[33]    --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --      
     //   --        --        --        --        --        --      pp_2[0]   pp_2[1]   pp_2[2]   pp_2[3]   pp_2[4]   pp_2[5]   pp_2[6]   pp_2[7]   pp_2[8]   pp_2[9]   pp_2[10]  pp_2[11]  pp_2[12]  pp_2[13]  pp_2[14]  pp_2[15]  pp_2[16]  pp_2[17]  pp_2[18]  pp_2[19]  pp_2[20]  pp_2[21]  pp_2[22]  pp_2[23]  pp_2[24]  pp_2[25]  pp_2[26]  pp_2[27]  pp_2[28]  pp_2[29]  pp_2[30]  pp_2[31]  pp_2[32]  pp_2[33]    --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --      
     //   --        --        --        --        --        --        --        --        --      pp_3[0]   pp_3[1]   pp_3[2]   pp_3[3]   pp_3[4]   pp_3[5]   pp_3[6]   pp_3[7]   pp_3[8]   pp_3[9]   pp_3[10]  pp_3[11]  pp_3[12]  pp_3[13]  pp_3[14]  pp_3[15]  pp_3[16]  pp_3[17]  pp_3[18]  pp_3[19]  pp_3[20]  pp_3[21]  pp_3[22]  pp_3[23]  pp_3[24]  pp_3[25]  pp_3[26]  pp_3[27]  pp_3[28]  pp_3[29]  pp_3[30]  pp_3[31]  pp_3[32]  pp_3[33]    --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --      
     //   --        --        --        --        --        --        --        --        --        --        --        --      pp_4[0]   pp_4[1]   pp_4[2]   pp_4[3]   pp_4[4]   pp_4[5]   pp_4[6]   pp_4[7]   pp_4[8]   pp_4[9]   pp_4[10]  pp_4[11]  pp_4[12]  pp_4[13]  pp_4[14]  pp_4[15]  pp_4[16]  pp_4[17]  pp_4[18]  pp_4[19]  pp_4[20]  pp_4[21]  pp_4[22]  pp_4[23]  pp_4[24]  pp_4[25]  pp_4[26]  pp_4[27]  pp_4[28]  pp_4[29]  pp_4[30]  pp_4[31]  pp_4[32]  pp_4[33]    --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --      
     //   --        --        --        --        --        --        --        --        --        --        --        --        --        --        --      pp_5[0]   pp_5[1]   pp_5[2]   pp_5[3]   pp_5[4]   pp_5[5]   pp_5[6]   pp_5[7]   pp_5[8]   pp_5[9]   pp_5[10]  pp_5[11]  pp_5[12]  pp_5[13]  pp_5[14]  pp_5[15]  pp_5[16]  pp_5[17]  pp_5[18]  pp_5[19]  pp_5[20]  pp_5[21]  pp_5[22]  pp_5[23]  pp_5[24]  pp_5[25]  pp_5[26]  pp_5[27]  pp_5[28]  pp_5[29]  pp_5[30]  pp_5[31]  pp_5[32]  pp_5[33]    --        --        --        --        --        --        --        --        --        --        --        --        --        --        --      
     //   --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --      pp_6[0]   pp_6[1]   pp_6[2]   pp_6[3]   pp_6[4]   pp_6[5]   pp_6[6]   pp_6[7]   pp_6[8]   pp_6[9]   pp_6[10]  pp_6[11]  pp_6[12]  pp_6[13]  pp_6[14]  pp_6[15]  pp_6[16]  pp_6[17]  pp_6[18]  pp_6[19]  pp_6[20]  pp_6[21]  pp_6[22]  pp_6[23]  pp_6[24]  pp_6[25]  pp_6[26]  pp_6[27]  pp_6[28]  pp_6[29]  pp_6[30]  pp_6[31]  pp_6[32]  pp_6[33]    --        --        --        --        --        --        --        --        --        --        --        --      
     //   --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --      pp_7[0]   pp_7[1]   pp_7[2]   pp_7[3]   pp_7[4]   pp_7[5]   pp_7[6]   pp_7[7]   pp_7[8]   pp_7[9]   pp_7[10]  pp_7[11]  pp_7[12]  pp_7[13]  pp_7[14]  pp_7[15]  pp_7[16]  pp_7[17]  pp_7[18]  pp_7[19]  pp_7[20]  pp_7[21]  pp_7[22]  pp_7[23]  pp_7[24]  pp_7[25]  pp_7[26]  pp_7[27]  pp_7[28]  pp_7[29]  pp_7[30]  pp_7[31]  pp_7[32]  pp_7[33]    --        --        --        --        --        --        --        --        --      
     //   --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --      pp_8[0]   pp_8[1]   pp_8[2]   pp_8[3]   pp_8[4]   pp_8[5]   pp_8[6]   pp_8[7]   pp_8[8]   pp_8[9]   pp_8[10]  pp_8[11]  pp_8[12]  pp_8[13]  pp_8[14]  pp_8[15]  pp_8[16]  pp_8[17]  pp_8[18]  pp_8[19]  pp_8[20]  pp_8[21]  pp_8[22]  pp_8[23]  pp_8[24]  pp_8[25]  pp_8[26]  pp_8[27]  pp_8[28]  pp_8[29]  pp_8[30]  pp_8[31]  pp_8[32]  pp_8[33]    --        --        --        --        --        --      
     //   --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --      pp_9[0]   pp_9[1]   pp_9[2]   pp_9[3]   pp_9[4]   pp_9[5]   pp_9[6]   pp_9[7]   pp_9[8]   pp_9[9]   pp_9[10]  pp_9[11]  pp_9[12]  pp_9[13]  pp_9[14]  pp_9[15]  pp_9[16]  pp_9[17]  pp_9[18]  pp_9[19]  pp_9[20]  pp_9[21]  pp_9[22]  pp_9[23]  pp_9[24]  pp_9[25]  pp_9[26]  pp_9[27]  pp_9[28]  pp_9[29]  pp_9[30]  pp_9[31]  pp_9[32]  pp_9[33]    --        --        --      
     //   --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --      pp_10[0]  pp_10[1]  pp_10[2]  pp_10[3]  pp_10[4]  pp_10[5]  pp_10[6]  pp_10[7]  pp_10[8]  pp_10[9]  pp_10[10] pp_10[11] pp_10[12] pp_10[13] pp_10[14] pp_10[15] pp_10[16] pp_10[17] pp_10[18] pp_10[19] pp_10[20] pp_10[21] pp_10[22] pp_10[23] pp_10[24] pp_10[25] pp_10[26] pp_10[27] pp_10[28] pp_10[29] pp_10[30] pp_10[31] pp_10[32] pp_10[33] 
     // tcomp0      --        --      tcomp1      --        --      tcomp2      --        --      tcomp3      --        --      tcomp4      --        --      tcomp5      --        --      tcomp6      --        --      tcomp7      --        --      tcomp8      --        --      tcomp9      --        --      tcomp10     --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --      
     //   --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --        --      
    
// Creating Summation Tree 

    
    // Dadda Summation Stage 1
    logic s0 ,c0;
    ha ha0 (pp_0[24], pp_1[21], s0, c0);
    logic s1 ,c1;
    ha ha1 (pp_0[25], pp_1[22], s1, c1);
    logic s2 ,c2;
    ha ha2 (pp_0[26], pp_1[23], s2, c2);
    logic s3 ,c3; 
    fa fa3 (pp_0[27], pp_1[24], pp_2[21], s3, c3);
    logic s4 ,c4;
    ha ha4 (pp_3[18], pp_4[15], s4, c4);
    logic s5 ,c5; 
    fa fa5 (pp_0[28], pp_1[25], pp_2[22], s5, c5);
    logic s6 ,c6;
    ha ha6 (pp_3[19], pp_4[16], s6, c6);
    logic s7 ,c7; 
    fa fa7 (pp_0[29], pp_1[26], pp_2[23], s7, c7);
    logic s8 ,c8;
    ha ha8 (pp_3[20], pp_4[17], s8, c8);
    logic s9 ,c9; 
    fa fa9 (pp_0[30], pp_1[27], pp_2[24], s9, c9);
    logic s10 ,c10; 
    fa fa10 (pp_3[21], pp_4[18], pp_5[15], s10, c10);
    logic s11 ,c11;
    ha ha11 (pp_6[12], pp_7[9], s11, c11);
    logic s12 ,c12; 
    fa fa12 (pp_0[31], pp_1[28], pp_2[25], s12, c12);
    logic s13 ,c13; 
    fa fa13 (pp_3[22], pp_4[19], pp_5[16], s13, c13);
    logic s14 ,c14;
    ha ha14 (pp_6[13], pp_7[10], s14, c14);
    logic s15 ,c15; 
    fa fa15 (pp_0[32], pp_1[29], pp_2[26], s15, c15);
    logic s16 ,c16; 
    fa fa16 (pp_3[23], pp_4[20], pp_5[17], s16, c16);
    logic s17 ,c17;
    ha ha17 (pp_6[14], pp_7[11], s17, c17);
    logic s18 ,c18; 
    fa fa18 (pp_0[33], pp_1[30], pp_2[27], s18, c18);
    logic s19 ,c19; 
    fa fa19 (pp_3[24], pp_4[21], pp_5[18], s19, c19);
    logic s20 ,c20;
    ha ha20 (pp_6[15], pp_7[12], s20, c20);
    logic s21 ,c21; 
    fa fa21 (pp_1[31], pp_2[28], pp_3[25], s21, c21);
    logic s22 ,c22; 
    fa fa22 (pp_4[22], pp_5[19], pp_6[16], s22, c22);
    logic s23 ,c23; 
    fa fa23 (pp_1[32], pp_2[29], pp_3[26], s23, c23);
    logic s24 ,c24;
    ha ha24 (pp_4[23], pp_5[20], s24, c24);
    logic s25 ,c25; 
    fa fa25 (pp_1[33], pp_2[30], pp_3[27], s25, c25);
    logic s26 ,c26;
    ha ha26 (pp_4[24], pp_5[21], s26, c26);
    logic s27 ,c27; 
    fa fa27 (pp_2[31], pp_3[28], pp_4[25], s27, c27);
    logic s28 ,c28;
    ha ha28 (pp_2[32], pp_3[29], s28, c28);
    logic s29 ,c29;
    ha ha29 (pp_2[33], pp_3[30], s29, c29);
    
    // Dadda Summation Stage 2
    logic s30 ,c30;
    ha ha30 (pp_0[15], pp_1[12], s30, c30);
    logic s31 ,c31;
    ha ha31 (pp_0[16], pp_1[13], s31, c31);
    logic s32 ,c32;
    ha ha32 (pp_0[17], pp_1[14], s32, c32);
    logic s33 ,c33; 
    fa fa33 (pp_0[18], pp_1[15], pp_2[12], s33, c33);
    logic s34 ,c34;
    ha ha34 (pp_3[9], pp_4[6], s34, c34);
    logic s35 ,c35; 
    fa fa35 (pp_0[19], pp_1[16], pp_2[13], s35, c35);
    logic s36 ,c36;
    ha ha36 (pp_3[10], pp_4[7], s36, c36);
    logic s37 ,c37; 
    fa fa37 (pp_0[20], pp_1[17], pp_2[14], s37, c37);
    logic s38 ,c38;
    ha ha38 (pp_3[11], pp_4[8], s38, c38);
    logic s39 ,c39; 
    fa fa39 (pp_0[21], pp_1[18], pp_2[15], s39, c39);
    logic s40 ,c40; 
    fa fa40 (pp_3[12], pp_4[9], pp_5[6], s40, c40);
    logic s41 ,c41;
    ha ha41 (pp_6[3], pp_7[0], s41, c41);
    logic s42 ,c42; 
    fa fa42 (pp_0[22], pp_1[19], pp_2[16], s42, c42);
    logic s43 ,c43; 
    fa fa43 (pp_3[13], pp_4[10], pp_5[7], s43, c43);
    logic s44 ,c44;
    ha ha44 (pp_6[4], pp_7[1], s44, c44);
    logic s45 ,c45; 
    fa fa45 (pp_0[23], pp_1[20], pp_2[17], s45, c45);
    logic s46 ,c46; 
    fa fa46 (pp_3[14], pp_4[11], pp_5[8], s46, c46);
    logic s47 ,c47;
    ha ha47 (pp_6[5], pp_7[2], s47, c47);
    logic s48 ,c48; 
    fa fa48 (pp_2[18], pp_3[15], pp_4[12], s48, c48);
    logic s49 ,c49; 
    fa fa49 (pp_5[9], pp_6[6], pp_7[3], s49, c49);
    logic s50 ,c50; 
    fa fa50 (pp_8[0], tcomp8, s0, s50, c50);
    logic s51 ,c51; 
    fa fa51 (pp_2[19], pp_3[16], pp_4[13], s51, c51);
    logic s52 ,c52; 
    fa fa52 (pp_5[10], pp_6[7], pp_7[4], s52, c52);
    logic s53 ,c53; 
    fa fa53 (pp_8[1], c0, s1, s53, c53);
    logic s54 ,c54; 
    fa fa54 (pp_2[20], pp_3[17], pp_4[14], s54, c54);
    logic s55 ,c55; 
    fa fa55 (pp_5[11], pp_6[8], pp_7[5], s55, c55);
    logic s56 ,c56; 
    fa fa56 (pp_8[2], c1, s2, s56, c56);
    logic s57 ,c57; 
    fa fa57 (pp_5[12], pp_6[9], pp_7[6], s57, c57);
    logic s58 ,c58; 
    fa fa58 (pp_8[3], pp_9[0], tcomp9, s58, c58);
    logic s59 ,c59; 
    fa fa59 (c2, s3, s4, s59, c59);
    logic s60 ,c60; 
    fa fa60 (pp_5[13], pp_6[10], pp_7[7], s60, c60);
    logic s61 ,c61; 
    fa fa61 (pp_8[4], pp_9[1], c3, s61, c61);
    logic s62 ,c62; 
    fa fa62 (c4, s5, s6, s62, c62);
    logic s63 ,c63; 
    fa fa63 (pp_5[14], pp_6[11], pp_7[8], s63, c63);
    logic s64 ,c64; 
    fa fa64 (pp_8[5], pp_9[2], c5, s64, c64);
    logic s65 ,c65; 
    fa fa65 (c6, s7, s8, s65, c65);
    logic s66 ,c66; 
    fa fa66 (pp_8[6], pp_9[3], pp_10[0], s66, c66);
    logic s67 ,c67; 
    fa fa67 (tcomp10, c7, c8, s67, c67);
    logic s68 ,c68; 
    fa fa68 (s9, s10, s11, s68, c68);
    logic s69 ,c69; 
    fa fa69 (pp_8[7], pp_9[4], pp_10[1], s69, c69);
    logic s70 ,c70; 
    fa fa70 (c9, c10, c11, s70, c70);
    logic s71 ,c71; 
    fa fa71 (s12, s13, s14, s71, c71);
    logic s72 ,c72; 
    fa fa72 (pp_8[8], pp_9[5], pp_10[2], s72, c72);
    logic s73 ,c73; 
    fa fa73 (c12, c13, c14, s73, c73);
    logic s74 ,c74; 
    fa fa74 (s15, s16, s17, s74, c74);
    logic s75 ,c75; 
    fa fa75 (pp_8[9], pp_9[6], pp_10[3], s75, c75);
    logic s76 ,c76; 
    fa fa76 (c15, c16, c17, s76, c76);
    logic s77 ,c77; 
    fa fa77 (s18, s19, s20, s77, c77);
    logic s78 ,c78; 
    fa fa78 (pp_7[13], pp_8[10], pp_9[7], s78, c78);
    logic s79 ,c79; 
    fa fa79 (pp_10[4], c18, c19, s79, c79);
    logic s80 ,c80; 
    fa fa80 (c20, s21, s22, s80, c80);
    logic s81 ,c81; 
    fa fa81 (pp_6[17], pp_7[14], pp_8[11], s81, c81);
    logic s82 ,c82; 
    fa fa82 (pp_9[8], pp_10[5], c21, s82, c82);
    logic s83 ,c83; 
    fa fa83 (c22, s23, s24, s83, c83);
    logic s84 ,c84; 
    fa fa84 (pp_6[18], pp_7[15], pp_8[12], s84, c84);
    logic s85 ,c85; 
    fa fa85 (pp_9[9], pp_10[6], c23, s85, c85);
    logic s86 ,c86; 
    fa fa86 (c24, s25, s26, s86, c86);
    logic s87 ,c87; 
    fa fa87 (pp_5[22], pp_6[19], pp_7[16], s87, c87);
    logic s88 ,c88; 
    fa fa88 (pp_8[13], pp_9[10], pp_10[7], s88, c88);
    logic s89 ,c89; 
    fa fa89 (c25, c26, s27, s89, c89);
    logic s90 ,c90; 
    fa fa90 (pp_4[26], pp_5[23], pp_6[20], s90, c90);
    logic s91 ,c91; 
    fa fa91 (pp_7[17], pp_8[14], pp_9[11], s91, c91);
    logic s92 ,c92; 
    fa fa92 (pp_10[8], c27, s28, s92, c92);
    logic s93 ,c93; 
    fa fa93 (pp_4[27], pp_5[24], pp_6[21], s93, c93);
    logic s94 ,c94; 
    fa fa94 (pp_7[18], pp_8[15], pp_9[12], s94, c94);
    logic s95 ,c95; 
    fa fa95 (pp_10[9], c28, s29, s95, c95);
    logic s96 ,c96; 
    fa fa96 (pp_3[31], pp_4[28], pp_5[25], s96, c96);
    logic s97 ,c97; 
    fa fa97 (pp_6[22], pp_7[19], pp_8[16], s97, c97);
    logic s98 ,c98; 
    fa fa98 (pp_9[13], pp_10[10], c29, s98, c98);
    logic s99 ,c99; 
    fa fa99 (pp_3[32], pp_4[29], pp_5[26], s99, c99);
    logic s100 ,c100; 
    fa fa100 (pp_6[23], pp_7[20], pp_8[17], s100, c100);
    logic s101 ,c101;
    ha ha101 (pp_9[14], pp_10[11], s101, c101);
    logic s102 ,c102; 
    fa fa102 (pp_3[33], pp_4[30], pp_5[27], s102, c102);
    logic s103 ,c103; 
    fa fa103 (pp_6[24], pp_7[21], pp_8[18], s103, c103);
    logic s104 ,c104;
    ha ha104 (pp_9[15], pp_10[12], s104, c104);
    logic s105 ,c105; 
    fa fa105 (pp_4[31], pp_5[28], pp_6[25], s105, c105);
    logic s106 ,c106; 
    fa fa106 (pp_7[22], pp_8[19], pp_9[16], s106, c106);
    logic s107 ,c107; 
    fa fa107 (pp_4[32], pp_5[29], pp_6[26], s107, c107);
    logic s108 ,c108;
    ha ha108 (pp_7[23], pp_8[20], s108, c108);
    logic s109 ,c109; 
    fa fa109 (pp_4[33], pp_5[30], pp_6[27], s109, c109);
    logic s110 ,c110;
    ha ha110 (pp_7[24], pp_8[21], s110, c110);
    logic s111 ,c111; 
    fa fa111 (pp_5[31], pp_6[28], pp_7[25], s111, c111);
    logic s112 ,c112;
    ha ha112 (pp_5[32], pp_6[29], s112, c112);
    logic s113 ,c113;
    ha ha113 (pp_5[33], pp_6[30], s113, c113);
    
    // Dadda Summation Stage 3
    logic s114 ,c114;
    ha ha114 (pp_0[9], pp_1[6], s114, c114);
    logic s115 ,c115;
    ha ha115 (pp_0[10], pp_1[7], s115, c115);
    logic s116 ,c116;
    ha ha116 (pp_0[11], pp_1[8], s116, c116);
    logic s117 ,c117; 
    fa fa117 (pp_0[12], pp_1[9], pp_2[6], s117, c117);
    logic s118 ,c118;
    ha ha118 (pp_3[3], pp_4[0], s118, c118);
    logic s119 ,c119; 
    fa fa119 (pp_0[13], pp_1[10], pp_2[7], s119, c119);
    logic s120 ,c120;
    ha ha120 (pp_3[4], pp_4[1], s120, c120);
    logic s121 ,c121; 
    fa fa121 (pp_0[14], pp_1[11], pp_2[8], s121, c121);
    logic s122 ,c122;
    ha ha122 (pp_3[5], pp_4[2], s122, c122);
    logic s123 ,c123; 
    fa fa123 (pp_2[9], pp_3[6], pp_4[3], s123, c123);
    logic s124 ,c124; 
    fa fa124 (pp_5[0], tcomp5, s30, s124, c124);
    logic s125 ,c125; 
    fa fa125 (pp_2[10], pp_3[7], pp_4[4], s125, c125);
    logic s126 ,c126; 
    fa fa126 (pp_5[1], c30, s31, s126, c126);
    logic s127 ,c127; 
    fa fa127 (pp_2[11], pp_3[8], pp_4[5], s127, c127);
    logic s128 ,c128; 
    fa fa128 (pp_5[2], c31, s32, s128, c128);
    logic s129 ,c129; 
    fa fa129 (pp_5[3], pp_6[0], tcomp6, s129, c129);
    logic s130 ,c130; 
    fa fa130 (c32, s33, s34, s130, c130);
    logic s131 ,c131; 
    fa fa131 (pp_5[4], pp_6[1], c33, s131, c131);
    logic s132 ,c132; 
    fa fa132 (c34, s35, s36, s132, c132);
    logic s133 ,c133; 
    fa fa133 (pp_5[5], pp_6[2], c35, s133, c133);
    logic s134 ,c134; 
    fa fa134 (c36, s37, s38, s134, c134);
    logic s135 ,c135; 
    fa fa135 (tcomp7, c37, c38, s135, c135);
    logic s136 ,c136; 
    fa fa136 (s39, s40, s41, s136, c136);
    logic s137 ,c137; 
    fa fa137 (c39, c40, c41, s137, c137);
    logic s138 ,c138; 
    fa fa138 (s42, s43, s44, s138, c138);
    logic s139 ,c139; 
    fa fa139 (c42, c43, c44, s139, c139);
    logic s140 ,c140; 
    fa fa140 (s45, s46, s47, s140, c140);
    logic s141 ,c141; 
    fa fa141 (c45, c46, c47, s141, c141);
    logic s142 ,c142; 
    fa fa142 (s48, s49, s50, s142, c142);
    logic s143 ,c143; 
    fa fa143 (c48, c49, c50, s143, c143);
    logic s144 ,c144; 
    fa fa144 (s51, s52, s53, s144, c144);
    logic s145 ,c145; 
    fa fa145 (c51, c52, c53, s145, c145);
    logic s146 ,c146; 
    fa fa146 (s54, s55, s56, s146, c146);
    logic s147 ,c147; 
    fa fa147 (c54, c55, c56, s147, c147);
    logic s148 ,c148; 
    fa fa148 (s57, s58, s59, s148, c148);
    logic s149 ,c149; 
    fa fa149 (c57, c58, c59, s149, c149);
    logic s150 ,c150; 
    fa fa150 (s60, s61, s62, s150, c150);
    logic s151 ,c151; 
    fa fa151 (c60, c61, c62, s151, c151);
    logic s152 ,c152; 
    fa fa152 (s63, s64, s65, s152, c152);
    logic s153 ,c153; 
    fa fa153 (c63, c64, c65, s153, c153);
    logic s154 ,c154; 
    fa fa154 (s66, s67, s68, s154, c154);
    logic s155 ,c155; 
    fa fa155 (c66, c67, c68, s155, c155);
    logic s156 ,c156; 
    fa fa156 (s69, s70, s71, s156, c156);
    logic s157 ,c157; 
    fa fa157 (c69, c70, c71, s157, c157);
    logic s158 ,c158; 
    fa fa158 (s72, s73, s74, s158, c158);
    logic s159 ,c159; 
    fa fa159 (c72, c73, c74, s159, c159);
    logic s160 ,c160; 
    fa fa160 (s75, s76, s77, s160, c160);
    logic s161 ,c161; 
    fa fa161 (c75, c76, c77, s161, c161);
    logic s162 ,c162; 
    fa fa162 (s78, s79, s80, s162, c162);
    logic s163 ,c163; 
    fa fa163 (c78, c79, c80, s163, c163);
    logic s164 ,c164; 
    fa fa164 (s81, s82, s83, s164, c164);
    logic s165 ,c165; 
    fa fa165 (c81, c82, c83, s165, c165);
    logic s166 ,c166; 
    fa fa166 (s84, s85, s86, s166, c166);
    logic s167 ,c167; 
    fa fa167 (c84, c85, c86, s167, c167);
    logic s168 ,c168; 
    fa fa168 (s87, s88, s89, s168, c168);
    logic s169 ,c169; 
    fa fa169 (c87, c88, c89, s169, c169);
    logic s170 ,c170; 
    fa fa170 (s90, s91, s92, s170, c170);
    logic s171 ,c171; 
    fa fa171 (c90, c91, c92, s171, c171);
    logic s172 ,c172; 
    fa fa172 (s93, s94, s95, s172, c172);
    logic s173 ,c173; 
    fa fa173 (c93, c94, c95, s173, c173);
    logic s174 ,c174; 
    fa fa174 (s96, s97, s98, s174, c174);
    logic s175 ,c175; 
    fa fa175 (c96, c97, c98, s175, c175);
    logic s176 ,c176; 
    fa fa176 (s99, s100, s101, s176, c176);
    logic s177 ,c177; 
    fa fa177 (c99, c100, c101, s177, c177);
    logic s178 ,c178; 
    fa fa178 (s102, s103, s104, s178, c178);
    logic s179 ,c179; 
    fa fa179 (pp_10[13], c102, c103, s179, c179);
    logic s180 ,c180; 
    fa fa180 (c104, s105, s106, s180, c180);
    logic s181 ,c181; 
    fa fa181 (pp_9[17], pp_10[14], c105, s181, c181);
    logic s182 ,c182; 
    fa fa182 (c106, s107, s108, s182, c182);
    logic s183 ,c183; 
    fa fa183 (pp_9[18], pp_10[15], c107, s183, c183);
    logic s184 ,c184; 
    fa fa184 (c108, s109, s110, s184, c184);
    logic s185 ,c185; 
    fa fa185 (pp_8[22], pp_9[19], pp_10[16], s185, c185);
    logic s186 ,c186; 
    fa fa186 (c109, c110, s111, s186, c186);
    logic s187 ,c187; 
    fa fa187 (pp_7[26], pp_8[23], pp_9[20], s187, c187);
    logic s188 ,c188; 
    fa fa188 (pp_10[17], c111, s112, s188, c188);
    logic s189 ,c189; 
    fa fa189 (pp_7[27], pp_8[24], pp_9[21], s189, c189);
    logic s190 ,c190; 
    fa fa190 (pp_10[18], c112, s113, s190, c190);
    logic s191 ,c191; 
    fa fa191 (pp_6[31], pp_7[28], pp_8[25], s191, c191);
    logic s192 ,c192; 
    fa fa192 (pp_9[22], pp_10[19], c113, s192, c192);
    logic s193 ,c193; 
    fa fa193 (pp_6[32], pp_7[29], pp_8[26], s193, c193);
    logic s194 ,c194;
    ha ha194 (pp_9[23], pp_10[20], s194, c194);
    logic s195 ,c195; 
    fa fa195 (pp_6[33], pp_7[30], pp_8[27], s195, c195);
    logic s196 ,c196;
    ha ha196 (pp_9[24], pp_10[21], s196, c196);
    logic s197 ,c197; 
    fa fa197 (pp_7[31], pp_8[28], pp_9[25], s197, c197);
    logic s198 ,c198;
    ha ha198 (pp_7[32], pp_8[29], s198, c198);
    logic s199 ,c199;
    ha ha199 (pp_7[33], pp_8[30], s199, c199);
    
    // Dadda Summation Stage 4
    logic s200 ,c200;
    ha ha200 (pp_0[6], pp_1[3], s200, c200);
    logic s201 ,c201;
    ha ha201 (pp_0[7], pp_1[4], s201, c201);
    logic s202 ,c202;
    ha ha202 (pp_0[8], pp_1[5], s202, c202);
    logic s203 ,c203; 
    fa fa203 (pp_2[3], pp_3[0], tcomp3, s203, c203);
    logic s204 ,c204; 
    fa fa204 (pp_2[4], pp_3[1], c114, s204, c204);
    logic s205 ,c205; 
    fa fa205 (pp_2[5], pp_3[2], c115, s205, c205);
    logic s206 ,c206; 
    fa fa206 (tcomp4, c116, s117, s206, c206);
    logic s207 ,c207; 
    fa fa207 (c117, c118, s119, s207, c207);
    logic s208 ,c208; 
    fa fa208 (c119, c120, s121, s208, c208);
    logic s209 ,c209; 
    fa fa209 (c121, c122, s123, s209, c209);
    logic s210 ,c210; 
    fa fa210 (c123, c124, s125, s210, c210);
    logic s211 ,c211; 
    fa fa211 (c125, c126, s127, s211, c211);
    logic s212 ,c212; 
    fa fa212 (c127, c128, s129, s212, c212);
    logic s213 ,c213; 
    fa fa213 (c129, c130, s131, s213, c213);
    logic s214 ,c214; 
    fa fa214 (c131, c132, s133, s214, c214);
    logic s215 ,c215; 
    fa fa215 (c133, c134, s135, s215, c215);
    logic s216 ,c216; 
    fa fa216 (c135, c136, s137, s216, c216);
    logic s217 ,c217; 
    fa fa217 (c137, c138, s139, s217, c217);
    logic s218 ,c218; 
    fa fa218 (c139, c140, s141, s218, c218);
    logic s219 ,c219; 
    fa fa219 (c141, c142, s143, s219, c219);
    logic s220 ,c220; 
    fa fa220 (c143, c144, s145, s220, c220);
    logic s221 ,c221; 
    fa fa221 (c145, c146, s147, s221, c221);
    logic s222 ,c222; 
    fa fa222 (c147, c148, s149, s222, c222);
    logic s223 ,c223; 
    fa fa223 (c149, c150, s151, s223, c223);
    logic s224 ,c224; 
    fa fa224 (c151, c152, s153, s224, c224);
    logic s225 ,c225; 
    fa fa225 (c153, c154, s155, s225, c225);
    logic s226 ,c226; 
    fa fa226 (c155, c156, s157, s226, c226);
    logic s227 ,c227; 
    fa fa227 (c157, c158, s159, s227, c227);
    logic s228 ,c228; 
    fa fa228 (c159, c160, s161, s228, c228);
    logic s229 ,c229; 
    fa fa229 (c161, c162, s163, s229, c229);
    logic s230 ,c230; 
    fa fa230 (c163, c164, s165, s230, c230);
    logic s231 ,c231; 
    fa fa231 (c165, c166, s167, s231, c231);
    logic s232 ,c232; 
    fa fa232 (c167, c168, s169, s232, c232);
    logic s233 ,c233; 
    fa fa233 (c169, c170, s171, s233, c233);
    logic s234 ,c234; 
    fa fa234 (c171, c172, s173, s234, c234);
    logic s235 ,c235; 
    fa fa235 (c173, c174, s175, s235, c235);
    logic s236 ,c236; 
    fa fa236 (c175, c176, s177, s236, c236);
    logic s237 ,c237; 
    fa fa237 (c177, c178, s179, s237, c237);
    logic s238 ,c238; 
    fa fa238 (c179, c180, s181, s238, c238);
    logic s239 ,c239; 
    fa fa239 (c181, c182, s183, s239, c239);
    logic s240 ,c240; 
    fa fa240 (c183, c184, s185, s240, c240);
    logic s241 ,c241; 
    fa fa241 (c185, c186, s187, s241, c241);
    logic s242 ,c242; 
    fa fa242 (c187, c188, s189, s242, c242);
    logic s243 ,c243; 
    fa fa243 (c189, c190, s191, s243, c243);
    logic s244 ,c244; 
    fa fa244 (c191, c192, s193, s244, c244);
    logic s245 ,c245; 
    fa fa245 (c193, c194, s195, s245, c245);
    logic s246 ,c246; 
    fa fa246 (pp_10[22], c195, c196, s246, c246);
    logic s247 ,c247; 
    fa fa247 (pp_9[26], pp_10[23], c197, s247, c247);
    logic s248 ,c248; 
    fa fa248 (pp_9[27], pp_10[24], c198, s248, c248);
    logic s249 ,c249; 
    fa fa249 (pp_8[31], pp_9[28], pp_10[25], s249, c249);
    logic s250 ,c250;
    ha ha250 (pp_8[32], pp_9[29], s250, c250);
    logic s251 ,c251;
    ha ha251 (pp_8[33], pp_9[30], s251, c251);
    
    // Dadda Summation Stage 5
    logic s252 ,c252;
    ha ha252 (pp_0[3], pp_1[0], s252, c252);
    logic s253 ,c253;
    ha ha253 (pp_0[4], pp_1[1], s253, c253);
    logic s254 ,c254;
    ha ha254 (pp_0[5], pp_1[2], s254, c254);
    logic s255 ,c255; 
    fa fa255 (pp_2[0], tcomp2, s200, s255, c255);
    logic s256 ,c256; 
    fa fa256 (pp_2[1], c200, s201, s256, c256);
    logic s257 ,c257; 
    fa fa257 (pp_2[2], c201, s202, s257, c257);
    logic s258 ,c258; 
    fa fa258 (s114, c202, s203, s258, c258);
    logic s259 ,c259; 
    fa fa259 (s115, c203, s204, s259, c259);
    logic s260 ,c260; 
    fa fa260 (s116, c204, s205, s260, c260);
    logic s261 ,c261; 
    fa fa261 (s118, c205, s206, s261, c261);
    logic s262 ,c262; 
    fa fa262 (s120, c206, s207, s262, c262);
    logic s263 ,c263; 
    fa fa263 (s122, c207, s208, s263, c263);
    logic s264 ,c264; 
    fa fa264 (s124, c208, s209, s264, c264);
    logic s265 ,c265; 
    fa fa265 (s126, c209, s210, s265, c265);
    logic s266 ,c266; 
    fa fa266 (s128, c210, s211, s266, c266);
    logic s267 ,c267; 
    fa fa267 (s130, c211, s212, s267, c267);
    logic s268 ,c268; 
    fa fa268 (s132, c212, s213, s268, c268);
    logic s269 ,c269; 
    fa fa269 (s134, c213, s214, s269, c269);
    logic s270 ,c270; 
    fa fa270 (s136, c214, s215, s270, c270);
    logic s271 ,c271; 
    fa fa271 (s138, c215, s216, s271, c271);
    logic s272 ,c272; 
    fa fa272 (s140, c216, s217, s272, c272);
    logic s273 ,c273; 
    fa fa273 (s142, c217, s218, s273, c273);
    logic s274 ,c274; 
    fa fa274 (s144, c218, s219, s274, c274);
    logic s275 ,c275; 
    fa fa275 (s146, c219, s220, s275, c275);
    logic s276 ,c276; 
    fa fa276 (s148, c220, s221, s276, c276);
    logic s277 ,c277; 
    fa fa277 (s150, c221, s222, s277, c277);
    logic s278 ,c278; 
    fa fa278 (s152, c222, s223, s278, c278);
    logic s279 ,c279; 
    fa fa279 (s154, c223, s224, s279, c279);
    logic s280 ,c280; 
    fa fa280 (s156, c224, s225, s280, c280);
    logic s281 ,c281; 
    fa fa281 (s158, c225, s226, s281, c281);
    logic s282 ,c282; 
    fa fa282 (s160, c226, s227, s282, c282);
    logic s283 ,c283; 
    fa fa283 (s162, c227, s228, s283, c283);
    logic s284 ,c284; 
    fa fa284 (s164, c228, s229, s284, c284);
    logic s285 ,c285; 
    fa fa285 (s166, c229, s230, s285, c285);
    logic s286 ,c286; 
    fa fa286 (s168, c230, s231, s286, c286);
    logic s287 ,c287; 
    fa fa287 (s170, c231, s232, s287, c287);
    logic s288 ,c288; 
    fa fa288 (s172, c232, s233, s288, c288);
    logic s289 ,c289; 
    fa fa289 (s174, c233, s234, s289, c289);
    logic s290 ,c290; 
    fa fa290 (s176, c234, s235, s290, c290);
    logic s291 ,c291; 
    fa fa291 (s178, c235, s236, s291, c291);
    logic s292 ,c292; 
    fa fa292 (s180, c236, s237, s292, c292);
    logic s293 ,c293; 
    fa fa293 (s182, c237, s238, s293, c293);
    logic s294 ,c294; 
    fa fa294 (s184, c238, s239, s294, c294);
    logic s295 ,c295; 
    fa fa295 (s186, c239, s240, s295, c295);
    logic s296 ,c296; 
    fa fa296 (s188, c240, s241, s296, c296);
    logic s297 ,c297; 
    fa fa297 (s190, c241, s242, s297, c297);
    logic s298 ,c298; 
    fa fa298 (s192, c242, s243, s298, c298);
    logic s299 ,c299; 
    fa fa299 (s194, c243, s244, s299, c299);
    logic s300 ,c300; 
    fa fa300 (s196, c244, s245, s300, c300);
    logic s301 ,c301; 
    fa fa301 (s197, c245, s246, s301, c301);
    logic s302 ,c302; 
    fa fa302 (s198, c246, s247, s302, c302);
    logic s303 ,c303; 
    fa fa303 (s199, c247, s248, s303, c303);
    logic s304 ,c304; 
    fa fa304 (c199, c248, s249, s304, c304);
    logic s305 ,c305; 
    fa fa305 (pp_10[26], c249, s250, s305, c305);
    logic s306 ,c306; 
    fa fa306 (pp_10[27], c250, s251, s306, c306);
    logic s307 ,c307; 
    fa fa307 (pp_9[31], pp_10[28], c251, s307, c307);
    logic s308 ,c308;
    ha ha308 (pp_9[32], pp_10[29], s308, c308);
    logic s309 ,c309;
    ha ha309 (pp_9[33], pp_10[30], s309, c309);
    
    
    assign result0[63:0] = {pp_10[33], pp_10[32], pp_10[31], c308, c307, c306, c305, c304, c303, c302, c301, c300, c299, c298, c297, c296, c295, c294, c293, c292, c291, c290, c289, c288, c287, c286, c285, c284, c283, c282, c281, c280, c279, c278, c277, c276, c275, c274, c273, c272, c271, c270, c269, c268, c267, c266, c265, c264, c263, c262, c261, c260, c259, c258, c257, c256, c255, c254, c253, c252, tcomp1, pp_0[2], pp_0[1], pp_0[0] };
    assign result1[63:0] = {1'b0, 1'b0, c309, s309, s308, s307, s306, s305, s304, s303, s302, s301, s300, s299, s298, s297, s296, s295, s294, s293, s292, s291, s290, s289, s288, s287, s286, s285, s284, s283, s282, s281, s280, s279, s278, s277, s276, s275, s274, s273, s272, s271, s270, s269, s268, s267, s266, s265, s264, s263, s262, s261, s260, s259, s258, s257, s256, s255, s254, s253, s252, 1'b0, 1'b0, tcomp0 };
endmodule



module HC_34_carry ( 
        input logic carryin,
        input logic [33:0] IN1,
        input logic [33:0] IN2,
        output logic [34:0] OUT);
    
    wire logic [33:0] p_0;
    wire logic [33:0] g_0;
    assign g_0[33:1] = IN1[33:1] & IN2[33:1];
    assign p_0[33:1] = IN1[33:1] ^ IN2[33:1];
    fa m0 (carryin, IN1[0], IN2[0], p_0[0], g_0[0]);
    
// Han-Carlson Adder 

    
    // HC stage 1
    wire logic p_1_1;
    wire logic g_1_1;
    assign p_1_1 = p_0[1] & p_0[0];
    assign g_1_1 = (p_0[1] & g_0[0]) | g_0[1];
    wire logic p_1_3;
    wire logic g_1_3;
    assign p_1_3 = p_0[3] & p_0[2];
    assign g_1_3 = (p_0[3] & g_0[2]) | g_0[3];
    wire logic p_1_5;
    wire logic g_1_5;
    assign p_1_5 = p_0[5] & p_0[4];
    assign g_1_5 = (p_0[5] & g_0[4]) | g_0[5];
    wire logic p_1_7;
    wire logic g_1_7;
    assign p_1_7 = p_0[7] & p_0[6];
    assign g_1_7 = (p_0[7] & g_0[6]) | g_0[7];
    wire logic p_1_9;
    wire logic g_1_9;
    assign p_1_9 = p_0[9] & p_0[8];
    assign g_1_9 = (p_0[9] & g_0[8]) | g_0[9];
    wire logic p_1_11;
    wire logic g_1_11;
    assign p_1_11 = p_0[11] & p_0[10];
    assign g_1_11 = (p_0[11] & g_0[10]) | g_0[11];
    wire logic p_1_13;
    wire logic g_1_13;
    assign p_1_13 = p_0[13] & p_0[12];
    assign g_1_13 = (p_0[13] & g_0[12]) | g_0[13];
    wire logic p_1_15;
    wire logic g_1_15;
    assign p_1_15 = p_0[15] & p_0[14];
    assign g_1_15 = (p_0[15] & g_0[14]) | g_0[15];
    wire logic p_1_17;
    wire logic g_1_17;
    assign p_1_17 = p_0[17] & p_0[16];
    assign g_1_17 = (p_0[17] & g_0[16]) | g_0[17];
    wire logic p_1_19;
    wire logic g_1_19;
    assign p_1_19 = p_0[19] & p_0[18];
    assign g_1_19 = (p_0[19] & g_0[18]) | g_0[19];
    wire logic p_1_21;
    wire logic g_1_21;
    assign p_1_21 = p_0[21] & p_0[20];
    assign g_1_21 = (p_0[21] & g_0[20]) | g_0[21];
    wire logic p_1_23;
    wire logic g_1_23;
    assign p_1_23 = p_0[23] & p_0[22];
    assign g_1_23 = (p_0[23] & g_0[22]) | g_0[23];
    wire logic p_1_25;
    wire logic g_1_25;
    assign p_1_25 = p_0[25] & p_0[24];
    assign g_1_25 = (p_0[25] & g_0[24]) | g_0[25];
    wire logic p_1_27;
    wire logic g_1_27;
    assign p_1_27 = p_0[27] & p_0[26];
    assign g_1_27 = (p_0[27] & g_0[26]) | g_0[27];
    wire logic p_1_29;
    wire logic g_1_29;
    assign p_1_29 = p_0[29] & p_0[28];
    assign g_1_29 = (p_0[29] & g_0[28]) | g_0[29];
    wire logic p_1_31;
    wire logic g_1_31;
    assign p_1_31 = p_0[31] & p_0[30];
    assign g_1_31 = (p_0[31] & g_0[30]) | g_0[31];
    wire logic p_1_33;
    wire logic g_1_33;
    assign p_1_33 = p_0[33] & p_0[32];
    assign g_1_33 = (p_0[33] & g_0[32]) | g_0[33];
    
    // HC stage 2
    wire logic p_2_3;
    wire logic g_2_3;
    assign p_2_3 = p_1_3 & p_1_1;
    assign g_2_3 = (p_1_3 & g_1_1) | g_1_3;
    wire logic p_2_5;
    wire logic g_2_5;
    assign p_2_5 = p_1_5 & p_1_3;
    assign g_2_5 = (p_1_5 & g_1_3) | g_1_5;
    wire logic p_2_7;
    wire logic g_2_7;
    assign p_2_7 = p_1_7 & p_1_5;
    assign g_2_7 = (p_1_7 & g_1_5) | g_1_7;
    wire logic p_2_9;
    wire logic g_2_9;
    assign p_2_9 = p_1_9 & p_1_7;
    assign g_2_9 = (p_1_9 & g_1_7) | g_1_9;
    wire logic p_2_11;
    wire logic g_2_11;
    assign p_2_11 = p_1_11 & p_1_9;
    assign g_2_11 = (p_1_11 & g_1_9) | g_1_11;
    wire logic p_2_13;
    wire logic g_2_13;
    assign p_2_13 = p_1_13 & p_1_11;
    assign g_2_13 = (p_1_13 & g_1_11) | g_1_13;
    wire logic p_2_15;
    wire logic g_2_15;
    assign p_2_15 = p_1_15 & p_1_13;
    assign g_2_15 = (p_1_15 & g_1_13) | g_1_15;
    wire logic p_2_17;
    wire logic g_2_17;
    assign p_2_17 = p_1_17 & p_1_15;
    assign g_2_17 = (p_1_17 & g_1_15) | g_1_17;
    wire logic p_2_19;
    wire logic g_2_19;
    assign p_2_19 = p_1_19 & p_1_17;
    assign g_2_19 = (p_1_19 & g_1_17) | g_1_19;
    wire logic p_2_21;
    wire logic g_2_21;
    assign p_2_21 = p_1_21 & p_1_19;
    assign g_2_21 = (p_1_21 & g_1_19) | g_1_21;
    wire logic p_2_23;
    wire logic g_2_23;
    assign p_2_23 = p_1_23 & p_1_21;
    assign g_2_23 = (p_1_23 & g_1_21) | g_1_23;
    wire logic p_2_25;
    wire logic g_2_25;
    assign p_2_25 = p_1_25 & p_1_23;
    assign g_2_25 = (p_1_25 & g_1_23) | g_1_25;
    wire logic p_2_27;
    wire logic g_2_27;
    assign p_2_27 = p_1_27 & p_1_25;
    assign g_2_27 = (p_1_27 & g_1_25) | g_1_27;
    wire logic p_2_29;
    wire logic g_2_29;
    assign p_2_29 = p_1_29 & p_1_27;
    assign g_2_29 = (p_1_29 & g_1_27) | g_1_29;
    wire logic p_2_31;
    wire logic g_2_31;
    assign p_2_31 = p_1_31 & p_1_29;
    assign g_2_31 = (p_1_31 & g_1_29) | g_1_31;
    wire logic p_2_33;
    wire logic g_2_33;
    assign p_2_33 = p_1_33 & p_1_31;
    assign g_2_33 = (p_1_33 & g_1_31) | g_1_33;
    
    // HC stage 3
    wire logic p_3_5;
    wire logic g_3_5;
    assign p_3_5 = p_2_5 & p_1_1;
    assign g_3_5 = (p_2_5 & g_1_1) | g_2_5;
    wire logic p_3_7;
    wire logic g_3_7;
    assign p_3_7 = p_2_7 & p_2_3;
    assign g_3_7 = (p_2_7 & g_2_3) | g_2_7;
    wire logic p_3_9;
    wire logic g_3_9;
    assign p_3_9 = p_2_9 & p_2_5;
    assign g_3_9 = (p_2_9 & g_2_5) | g_2_9;
    wire logic p_3_11;
    wire logic g_3_11;
    assign p_3_11 = p_2_11 & p_2_7;
    assign g_3_11 = (p_2_11 & g_2_7) | g_2_11;
    wire logic p_3_13;
    wire logic g_3_13;
    assign p_3_13 = p_2_13 & p_2_9;
    assign g_3_13 = (p_2_13 & g_2_9) | g_2_13;
    wire logic p_3_15;
    wire logic g_3_15;
    assign p_3_15 = p_2_15 & p_2_11;
    assign g_3_15 = (p_2_15 & g_2_11) | g_2_15;
    wire logic p_3_17;
    wire logic g_3_17;
    assign p_3_17 = p_2_17 & p_2_13;
    assign g_3_17 = (p_2_17 & g_2_13) | g_2_17;
    wire logic p_3_19;
    wire logic g_3_19;
    assign p_3_19 = p_2_19 & p_2_15;
    assign g_3_19 = (p_2_19 & g_2_15) | g_2_19;
    wire logic p_3_21;
    wire logic g_3_21;
    assign p_3_21 = p_2_21 & p_2_17;
    assign g_3_21 = (p_2_21 & g_2_17) | g_2_21;
    wire logic p_3_23;
    wire logic g_3_23;
    assign p_3_23 = p_2_23 & p_2_19;
    assign g_3_23 = (p_2_23 & g_2_19) | g_2_23;
    wire logic p_3_25;
    wire logic g_3_25;
    assign p_3_25 = p_2_25 & p_2_21;
    assign g_3_25 = (p_2_25 & g_2_21) | g_2_25;
    wire logic p_3_27;
    wire logic g_3_27;
    assign p_3_27 = p_2_27 & p_2_23;
    assign g_3_27 = (p_2_27 & g_2_23) | g_2_27;
    wire logic p_3_29;
    wire logic g_3_29;
    assign p_3_29 = p_2_29 & p_2_25;
    assign g_3_29 = (p_2_29 & g_2_25) | g_2_29;
    wire logic p_3_31;
    wire logic g_3_31;
    assign p_3_31 = p_2_31 & p_2_27;
    assign g_3_31 = (p_2_31 & g_2_27) | g_2_31;
    wire logic p_3_33;
    wire logic g_3_33;
    assign p_3_33 = p_2_33 & p_2_29;
    assign g_3_33 = (p_2_33 & g_2_29) | g_2_33;
    
    // HC stage 4
    wire logic p_4_9;
    wire logic g_4_9;
    assign p_4_9 = p_3_9 & p_1_1;
    assign g_4_9 = (p_3_9 & g_1_1) | g_3_9;
    wire logic p_4_11;
    wire logic g_4_11;
    assign p_4_11 = p_3_11 & p_2_3;
    assign g_4_11 = (p_3_11 & g_2_3) | g_3_11;
    wire logic p_4_13;
    wire logic g_4_13;
    assign p_4_13 = p_3_13 & p_3_5;
    assign g_4_13 = (p_3_13 & g_3_5) | g_3_13;
    wire logic p_4_15;
    wire logic g_4_15;
    assign p_4_15 = p_3_15 & p_3_7;
    assign g_4_15 = (p_3_15 & g_3_7) | g_3_15;
    wire logic p_4_17;
    wire logic g_4_17;
    assign p_4_17 = p_3_17 & p_3_9;
    assign g_4_17 = (p_3_17 & g_3_9) | g_3_17;
    wire logic p_4_19;
    wire logic g_4_19;
    assign p_4_19 = p_3_19 & p_3_11;
    assign g_4_19 = (p_3_19 & g_3_11) | g_3_19;
    wire logic p_4_21;
    wire logic g_4_21;
    assign p_4_21 = p_3_21 & p_3_13;
    assign g_4_21 = (p_3_21 & g_3_13) | g_3_21;
    wire logic p_4_23;
    wire logic g_4_23;
    assign p_4_23 = p_3_23 & p_3_15;
    assign g_4_23 = (p_3_23 & g_3_15) | g_3_23;
    wire logic p_4_25;
    wire logic g_4_25;
    assign p_4_25 = p_3_25 & p_3_17;
    assign g_4_25 = (p_3_25 & g_3_17) | g_3_25;
    wire logic p_4_27;
    wire logic g_4_27;
    assign p_4_27 = p_3_27 & p_3_19;
    assign g_4_27 = (p_3_27 & g_3_19) | g_3_27;
    wire logic p_4_29;
    wire logic g_4_29;
    assign p_4_29 = p_3_29 & p_3_21;
    assign g_4_29 = (p_3_29 & g_3_21) | g_3_29;
    wire logic p_4_31;
    wire logic g_4_31;
    assign p_4_31 = p_3_31 & p_3_23;
    assign g_4_31 = (p_3_31 & g_3_23) | g_3_31;
    wire logic p_4_33;
    wire logic g_4_33;
    assign p_4_33 = p_3_33 & p_3_25;
    assign g_4_33 = (p_3_33 & g_3_25) | g_3_33;
    
    // HC stage 5
    wire logic p_5_17;
    wire logic g_5_17;
    assign p_5_17 = p_4_17 & p_1_1;
    assign g_5_17 = (p_4_17 & g_1_1) | g_4_17;
    wire logic p_5_19;
    wire logic g_5_19;
    assign p_5_19 = p_4_19 & p_2_3;
    assign g_5_19 = (p_4_19 & g_2_3) | g_4_19;
    wire logic p_5_21;
    wire logic g_5_21;
    assign p_5_21 = p_4_21 & p_3_5;
    assign g_5_21 = (p_4_21 & g_3_5) | g_4_21;
    wire logic p_5_23;
    wire logic g_5_23;
    assign p_5_23 = p_4_23 & p_3_7;
    assign g_5_23 = (p_4_23 & g_3_7) | g_4_23;
    wire logic p_5_25;
    wire logic g_5_25;
    assign p_5_25 = p_4_25 & p_4_9;
    assign g_5_25 = (p_4_25 & g_4_9) | g_4_25;
    wire logic p_5_27;
    wire logic g_5_27;
    assign p_5_27 = p_4_27 & p_4_11;
    assign g_5_27 = (p_4_27 & g_4_11) | g_4_27;
    wire logic p_5_29;
    wire logic g_5_29;
    assign p_5_29 = p_4_29 & p_4_13;
    assign g_5_29 = (p_4_29 & g_4_13) | g_4_29;
    wire logic p_5_31;
    wire logic g_5_31;
    assign p_5_31 = p_4_31 & p_4_15;
    assign g_5_31 = (p_4_31 & g_4_15) | g_4_31;
    wire logic p_5_33;
    wire logic g_5_33;
    assign p_5_33 = p_4_33 & p_4_17;
    assign g_5_33 = (p_4_33 & g_4_17) | g_4_33;
    
    // HC stage 6
    wire logic p_6_33;
    wire logic g_6_33;
    assign p_6_33 = p_5_33 & p_1_1;
    assign g_6_33 = (p_5_33 & g_1_1) | g_5_33;
    
    // HC stage 7
    wire logic p_7_2;
    wire logic g_7_2;
    assign p_7_2 = p_0[2] & p_1_1;
    assign g_7_2 = (p_0[2] & g_1_1) | g_0[2];
    wire logic p_7_4;
    wire logic g_7_4;
    assign p_7_4 = p_0[4] & p_2_3;
    assign g_7_4 = (p_0[4] & g_2_3) | g_0[4];
    wire logic p_7_6;
    wire logic g_7_6;
    assign p_7_6 = p_0[6] & p_3_5;
    assign g_7_6 = (p_0[6] & g_3_5) | g_0[6];
    wire logic p_7_8;
    wire logic g_7_8;
    assign p_7_8 = p_0[8] & p_3_7;
    assign g_7_8 = (p_0[8] & g_3_7) | g_0[8];
    wire logic p_7_10;
    wire logic g_7_10;
    assign p_7_10 = p_0[10] & p_4_9;
    assign g_7_10 = (p_0[10] & g_4_9) | g_0[10];
    wire logic p_7_12;
    wire logic g_7_12;
    assign p_7_12 = p_0[12] & p_4_11;
    assign g_7_12 = (p_0[12] & g_4_11) | g_0[12];
    wire logic p_7_14;
    wire logic g_7_14;
    assign p_7_14 = p_0[14] & p_4_13;
    assign g_7_14 = (p_0[14] & g_4_13) | g_0[14];
    wire logic p_7_16;
    wire logic g_7_16;
    assign p_7_16 = p_0[16] & p_4_15;
    assign g_7_16 = (p_0[16] & g_4_15) | g_0[16];
    wire logic p_7_18;
    wire logic g_7_18;
    assign p_7_18 = p_0[18] & p_5_17;
    assign g_7_18 = (p_0[18] & g_5_17) | g_0[18];
    wire logic p_7_20;
    wire logic g_7_20;
    assign p_7_20 = p_0[20] & p_5_19;
    assign g_7_20 = (p_0[20] & g_5_19) | g_0[20];
    wire logic p_7_22;
    wire logic g_7_22;
    assign p_7_22 = p_0[22] & p_5_21;
    assign g_7_22 = (p_0[22] & g_5_21) | g_0[22];
    wire logic p_7_24;
    wire logic g_7_24;
    assign p_7_24 = p_0[24] & p_5_23;
    assign g_7_24 = (p_0[24] & g_5_23) | g_0[24];
    wire logic p_7_26;
    wire logic g_7_26;
    assign p_7_26 = p_0[26] & p_5_25;
    assign g_7_26 = (p_0[26] & g_5_25) | g_0[26];
    wire logic p_7_28;
    wire logic g_7_28;
    assign p_7_28 = p_0[28] & p_5_27;
    assign g_7_28 = (p_0[28] & g_5_27) | g_0[28];
    wire logic p_7_30;
    wire logic g_7_30;
    assign p_7_30 = p_0[30] & p_5_29;
    assign g_7_30 = (p_0[30] & g_5_29) | g_0[30];
    wire logic p_7_32;
    wire logic g_7_32;
    assign p_7_32 = p_0[32] & p_5_31;
    assign g_7_32 = (p_0[32] & g_5_31) | g_0[32];
    
    // HC postprocess 
    assign OUT[0] = p_0[0];
    assign OUT[1] = p_0[1] ^ g_0[0];
    assign OUT[2] = p_0[2] ^ g_1_1;
    assign OUT[3] = p_0[3] ^ g_7_2;
    assign OUT[4] = p_0[4] ^ g_2_3;
    assign OUT[5] = p_0[5] ^ g_7_4;
    assign OUT[6] = p_0[6] ^ g_3_5;
    assign OUT[7] = p_0[7] ^ g_7_6;
    assign OUT[8] = p_0[8] ^ g_3_7;
    assign OUT[9] = p_0[9] ^ g_7_8;
    assign OUT[10] = p_0[10] ^ g_4_9;
    assign OUT[11] = p_0[11] ^ g_7_10;
    assign OUT[12] = p_0[12] ^ g_4_11;
    assign OUT[13] = p_0[13] ^ g_7_12;
    assign OUT[14] = p_0[14] ^ g_4_13;
    assign OUT[15] = p_0[15] ^ g_7_14;
    assign OUT[16] = p_0[16] ^ g_4_15;
    assign OUT[17] = p_0[17] ^ g_7_16;
    assign OUT[18] = p_0[18] ^ g_5_17;
    assign OUT[19] = p_0[19] ^ g_7_18;
    assign OUT[20] = p_0[20] ^ g_5_19;
    assign OUT[21] = p_0[21] ^ g_7_20;
    assign OUT[22] = p_0[22] ^ g_5_21;
    assign OUT[23] = p_0[23] ^ g_7_22;
    assign OUT[24] = p_0[24] ^ g_5_23;
    assign OUT[25] = p_0[25] ^ g_7_24;
    assign OUT[26] = p_0[26] ^ g_5_25;
    assign OUT[27] = p_0[27] ^ g_7_26;
    assign OUT[28] = p_0[28] ^ g_5_27;
    assign OUT[29] = p_0[29] ^ g_7_28;
    assign OUT[30] = p_0[30] ^ g_5_29;
    assign OUT[31] = p_0[31] ^ g_7_30;
    assign OUT[32] = p_0[32] ^ g_5_31;
    assign OUT[33] = p_0[33] ^ g_7_32;
    assign OUT[34] = g_6_33;
endmodule

module HC_34_carry_spec (
        input logic carryin,
        input logic [33:0] IN1,
        input logic [33:0] IN2,
        output logic adder_correct,
        output logic [34:0] spec_res);
    
    assign spec_res = IN1 + IN2 + carryin;
    wire [34:0] adder_res;
    HC_34_carry adder(carryin, IN1, IN2, adder_res);
    assign adder_correct = ((spec_res == adder_res) ? 1 : 0);
    
endmodule



module MAC_DT_SB8_HC_32x32_plus_32_63to0(
        input logic [31:0] IN1,
        input logic [31:0] IN2,
        input logic [31:0] IN3,
        output logic [63:0] result);
    wire logic [63:0] m1_0;
    wire logic [63:0] m1_1;
    DT_SB8_32x32 m1 (IN1[31:0], IN2[31:0], m1_0, m1_1);
    
    
    // The values to be summed in the summation tree, from LSB (left) to MSB:
     // m1_0[0]  m1_0[1]  m1_0[2]  m1_0[3]  m1_0[4]  m1_0[5]  m1_0[6]  m1_0[7]  m1_0[8]  m1_0[9]  m1_0[10] m1_0[11] m1_0[12] m1_0[13] m1_0[14] m1_0[15] m1_0[16] m1_0[17] m1_0[18] m1_0[19] m1_0[20] m1_0[21] m1_0[22] m1_0[23] m1_0[24] m1_0[25] m1_0[26] m1_0[27] m1_0[28] m1_0[29] m1_0[30] m1_0[31] m1_0[32] m1_0[33] m1_0[34] m1_0[35] m1_0[36] m1_0[37] m1_0[38] m1_0[39] m1_0[40] m1_0[41] m1_0[42] m1_0[43] m1_0[44] m1_0[45] m1_0[46] m1_0[47] m1_0[48] m1_0[49] m1_0[50] m1_0[51] m1_0[52] m1_0[53] m1_0[54] m1_0[55] m1_0[56] m1_0[57] m1_0[58] m1_0[59] m1_0[60] m1_0[61] m1_0[62] m1_0[63] 
     // m1_1[0]    --       --     m1_1[3]  m1_1[4]  m1_1[5]  m1_1[6]  m1_1[7]  m1_1[8]  m1_1[9]  m1_1[10] m1_1[11] m1_1[12] m1_1[13] m1_1[14] m1_1[15] m1_1[16] m1_1[17] m1_1[18] m1_1[19] m1_1[20] m1_1[21] m1_1[22] m1_1[23] m1_1[24] m1_1[25] m1_1[26] m1_1[27] m1_1[28] m1_1[29] m1_1[30] m1_1[31] m1_1[32] m1_1[33] m1_1[34] m1_1[35] m1_1[36] m1_1[37] m1_1[38] m1_1[39] m1_1[40] m1_1[41] m1_1[42] m1_1[43] m1_1[44] m1_1[45] m1_1[46] m1_1[47] m1_1[48] m1_1[49] m1_1[50] m1_1[51] m1_1[52] m1_1[53] m1_1[54] m1_1[55] m1_1[56] m1_1[57] m1_1[58] m1_1[59] m1_1[60] m1_1[61]   --       --     
     // IN3[0]   IN3[1]   IN3[2]   IN3[3]   IN3[4]   IN3[5]   IN3[6]   IN3[7]   IN3[8]   IN3[9]   IN3[10]  IN3[11]  IN3[12]  IN3[13]  IN3[14]  IN3[15]  IN3[16]  IN3[17]  IN3[18]  IN3[19]  IN3[20]  IN3[21]  IN3[22]  IN3[23]  IN3[24]  IN3[25]  IN3[26]  IN3[27]  IN3[28]  IN3[29]  IN3[30]  ~IN3[31]   --       --       --       --       --       --       --       --       --       --       --       --       --       --       --       --       --       --       --       --       --       --       --       --       --       --       --       --       --       --       --       --     
     //   --       --       --       --       --       --       --       --       --       --       --       --       --       --       --       --       --       --       --       --       --       --       --       --       --       --       --       --       --       --       --     1'b1     1'b1       --     1'b1     1'b1       --     1'b1     1'b1       --     1'b1     1'b1       --     1'b1     1'b1       --     1'b1     1'b1       --     1'b1     1'b1       --     1'b1     1'b1       --     1'b1     1'b1       --     1'b1     1'b1       --     1'b1     1'b1       --     
    
    // Dadda Summation Stage 1
    logic s0 ,c0;
    ha ha0 (m1_0[31], m1_1[31], s0, c0);
    logic s1 ,c1;
    ha ha1 (m1_0[32], m1_1[32], s1, c1);
    
    // Dadda Summation Stage 2
    logic s2 ,c2;
    ha ha2 (m1_0[0], m1_1[0], s2, c2);
    logic s3 ,c3;
    ha ha3 (m1_0[1], IN3[1], s3, c3);
    logic s4 ,c4;
    ha ha4 (m1_0[2], IN3[2], s4, c4);
    logic s5 ,c5; 
    fa fa5 (m1_0[3], m1_1[3], IN3[3], s5, c5);
    logic s6 ,c6; 
    fa fa6 (m1_0[4], m1_1[4], IN3[4], s6, c6);
    logic s7 ,c7; 
    fa fa7 (m1_0[5], m1_1[5], IN3[5], s7, c7);
    logic s8 ,c8; 
    fa fa8 (m1_0[6], m1_1[6], IN3[6], s8, c8);
    logic s9 ,c9; 
    fa fa9 (m1_0[7], m1_1[7], IN3[7], s9, c9);
    logic s10 ,c10; 
    fa fa10 (m1_0[8], m1_1[8], IN3[8], s10, c10);
    logic s11 ,c11; 
    fa fa11 (m1_0[9], m1_1[9], IN3[9], s11, c11);
    logic s12 ,c12; 
    fa fa12 (m1_0[10], m1_1[10], IN3[10], s12, c12);
    logic s13 ,c13; 
    fa fa13 (m1_0[11], m1_1[11], IN3[11], s13, c13);
    logic s14 ,c14; 
    fa fa14 (m1_0[12], m1_1[12], IN3[12], s14, c14);
    logic s15 ,c15; 
    fa fa15 (m1_0[13], m1_1[13], IN3[13], s15, c15);
    logic s16 ,c16; 
    fa fa16 (m1_0[14], m1_1[14], IN3[14], s16, c16);
    logic s17 ,c17; 
    fa fa17 (m1_0[15], m1_1[15], IN3[15], s17, c17);
    logic s18 ,c18; 
    fa fa18 (m1_0[16], m1_1[16], IN3[16], s18, c18);
    logic s19 ,c19; 
    fa fa19 (m1_0[17], m1_1[17], IN3[17], s19, c19);
    logic s20 ,c20; 
    fa fa20 (m1_0[18], m1_1[18], IN3[18], s20, c20);
    logic s21 ,c21; 
    fa fa21 (m1_0[19], m1_1[19], IN3[19], s21, c21);
    logic s22 ,c22; 
    fa fa22 (m1_0[20], m1_1[20], IN3[20], s22, c22);
    logic s23 ,c23; 
    fa fa23 (m1_0[21], m1_1[21], IN3[21], s23, c23);
    logic s24 ,c24; 
    fa fa24 (m1_0[22], m1_1[22], IN3[22], s24, c24);
    logic s25 ,c25; 
    fa fa25 (m1_0[23], m1_1[23], IN3[23], s25, c25);
    logic s26 ,c26; 
    fa fa26 (m1_0[24], m1_1[24], IN3[24], s26, c26);
    logic s27 ,c27; 
    fa fa27 (m1_0[25], m1_1[25], IN3[25], s27, c27);
    logic s28 ,c28; 
    fa fa28 (m1_0[26], m1_1[26], IN3[26], s28, c28);
    logic s29 ,c29; 
    fa fa29 (m1_0[27], m1_1[27], IN3[27], s29, c29);
    logic s30 ,c30; 
    fa fa30 (m1_0[28], m1_1[28], IN3[28], s30, c30);
    logic s31 ,c31; 
    fa fa31 (m1_0[29], m1_1[29], IN3[29], s31, c31);
    logic s32 ,c32; 
    fa fa32 (m1_0[30], m1_1[30], IN3[30], s32, c32);
    logic s33 ,c33; 
    fa fa33 (~IN3[31], 1'b1, s0, s33, c33);
    logic s34 ,c34; 
    fa fa34 (1'b1, c0, s1, s34, c34);
    logic s35 ,c35; 
    fa fa35 (m1_0[33], m1_1[33], c1, s35, c35);
    logic s36 ,c36; 
    fa fa36 (m1_0[34], m1_1[34], 1'b1, s36, c36);
    logic s37 ,c37; 
    fa fa37 (m1_0[35], m1_1[35], 1'b1, s37, c37);
    logic s38 ,c38;
    ha ha38 (m1_0[36], m1_1[36], s38, c38);
    logic s39 ,c39; 
    fa fa39 (m1_0[37], m1_1[37], 1'b1, s39, c39);
    logic s40 ,c40; 
    fa fa40 (m1_0[38], m1_1[38], 1'b1, s40, c40);
    logic s41 ,c41;
    ha ha41 (m1_0[39], m1_1[39], s41, c41);
    logic s42 ,c42; 
    fa fa42 (m1_0[40], m1_1[40], 1'b1, s42, c42);
    logic s43 ,c43; 
    fa fa43 (m1_0[41], m1_1[41], 1'b1, s43, c43);
    logic s44 ,c44;
    ha ha44 (m1_0[42], m1_1[42], s44, c44);
    logic s45 ,c45; 
    fa fa45 (m1_0[43], m1_1[43], 1'b1, s45, c45);
    logic s46 ,c46; 
    fa fa46 (m1_0[44], m1_1[44], 1'b1, s46, c46);
    logic s47 ,c47;
    ha ha47 (m1_0[45], m1_1[45], s47, c47);
    logic s48 ,c48; 
    fa fa48 (m1_0[46], m1_1[46], 1'b1, s48, c48);
    logic s49 ,c49; 
    fa fa49 (m1_0[47], m1_1[47], 1'b1, s49, c49);
    logic s50 ,c50;
    ha ha50 (m1_0[48], m1_1[48], s50, c50);
    logic s51 ,c51; 
    fa fa51 (m1_0[49], m1_1[49], 1'b1, s51, c51);
    logic s52 ,c52; 
    fa fa52 (m1_0[50], m1_1[50], 1'b1, s52, c52);
    logic s53 ,c53;
    ha ha53 (m1_0[51], m1_1[51], s53, c53);
    logic s54 ,c54; 
    fa fa54 (m1_0[52], m1_1[52], 1'b1, s54, c54);
    logic s55 ,c55; 
    fa fa55 (m1_0[53], m1_1[53], 1'b1, s55, c55);
    logic s56 ,c56;
    ha ha56 (m1_0[54], m1_1[54], s56, c56);
    logic s57 ,c57; 
    fa fa57 (m1_0[55], m1_1[55], 1'b1, s57, c57);
    logic s58 ,c58; 
    fa fa58 (m1_0[56], m1_1[56], 1'b1, s58, c58);
    logic s59 ,c59;
    ha ha59 (m1_0[57], m1_1[57], s59, c59);
    logic s60 ,c60; 
    fa fa60 (m1_0[58], m1_1[58], 1'b1, s60, c60);
    logic s61 ,c61; 
    fa fa61 (m1_0[59], m1_1[59], 1'b1, s61, c61);
    logic s62 ,c62;
    ha ha62 (m1_0[60], m1_1[60], s62, c62);
    logic s63 ,c63; 
    fa fa63 (m1_0[61], m1_1[61], 1'b1, s63, c63);
    logic s64 ,c64;
    ha ha64 (m1_0[62], 1'b1, s64, c64);
    
    logic [64:0] adder_result;
    HC_64 final_adder ({m1_0[63], c63, c62, c61, c60, c59, c58, c57, c56, c55, c54, c53, c52, c51, c50, c49, c48, c47, c46, c45, c44, c43, c42, c41, c40, c39, c38, c37, c36, c35, c34, c33, c32, c31, c30, c29, c28, c27, c26, c25, c24, c23, c22, c21, c20, c19, c18, c17, c16, c15, c14, c13, c12, c11, c10, c9, c8, c7, c6, c5, c4, c3, c2, IN3[0] }, {c64, s64, s63, s62, s61, s60, s59, s58, s57, s56, s55, s54, s53, s52, s51, s50, s49, s48, s47, s46, s45, s44, s43, s42, s41, s40, s39, s38, s37, s36, s35, s34, s33, s32, s31, s30, s29, s28, s27, s26, s25, s24, s23, s22, s21, s20, s19, s18, s17, s16, s15, s14, s13, s12, s11, s10, s9, s8, s7, s6, s5, s4, s3, s2 }, adder_result );
    assign result[63:0] = adder_result[63:0];

endmodule


module HC_64 ( 
        input logic [63:0] IN1,
        input logic [63:0] IN2,
        output logic [64:0] OUT);
    
    wire logic [63:0] p_0;
    wire logic [63:0] g_0;
    assign g_0 = IN1 & IN2;
    assign p_0 = IN1 ^ IN2;
    
// Han-Carlson Adder 

    
    // HC stage 1
    wire logic p_1_1;
    wire logic g_1_1;
    assign p_1_1 = p_0[1] & p_0[0];
    assign g_1_1 = (p_0[1] & g_0[0]) | g_0[1];
    wire logic p_1_3;
    wire logic g_1_3;
    assign p_1_3 = p_0[3] & p_0[2];
    assign g_1_3 = (p_0[3] & g_0[2]) | g_0[3];
    wire logic p_1_5;
    wire logic g_1_5;
    assign p_1_5 = p_0[5] & p_0[4];
    assign g_1_5 = (p_0[5] & g_0[4]) | g_0[5];
    wire logic p_1_7;
    wire logic g_1_7;
    assign p_1_7 = p_0[7] & p_0[6];
    assign g_1_7 = (p_0[7] & g_0[6]) | g_0[7];
    wire logic p_1_9;
    wire logic g_1_9;
    assign p_1_9 = p_0[9] & p_0[8];
    assign g_1_9 = (p_0[9] & g_0[8]) | g_0[9];
    wire logic p_1_11;
    wire logic g_1_11;
    assign p_1_11 = p_0[11] & p_0[10];
    assign g_1_11 = (p_0[11] & g_0[10]) | g_0[11];
    wire logic p_1_13;
    wire logic g_1_13;
    assign p_1_13 = p_0[13] & p_0[12];
    assign g_1_13 = (p_0[13] & g_0[12]) | g_0[13];
    wire logic p_1_15;
    wire logic g_1_15;
    assign p_1_15 = p_0[15] & p_0[14];
    assign g_1_15 = (p_0[15] & g_0[14]) | g_0[15];
    wire logic p_1_17;
    wire logic g_1_17;
    assign p_1_17 = p_0[17] & p_0[16];
    assign g_1_17 = (p_0[17] & g_0[16]) | g_0[17];
    wire logic p_1_19;
    wire logic g_1_19;
    assign p_1_19 = p_0[19] & p_0[18];
    assign g_1_19 = (p_0[19] & g_0[18]) | g_0[19];
    wire logic p_1_21;
    wire logic g_1_21;
    assign p_1_21 = p_0[21] & p_0[20];
    assign g_1_21 = (p_0[21] & g_0[20]) | g_0[21];
    wire logic p_1_23;
    wire logic g_1_23;
    assign p_1_23 = p_0[23] & p_0[22];
    assign g_1_23 = (p_0[23] & g_0[22]) | g_0[23];
    wire logic p_1_25;
    wire logic g_1_25;
    assign p_1_25 = p_0[25] & p_0[24];
    assign g_1_25 = (p_0[25] & g_0[24]) | g_0[25];
    wire logic p_1_27;
    wire logic g_1_27;
    assign p_1_27 = p_0[27] & p_0[26];
    assign g_1_27 = (p_0[27] & g_0[26]) | g_0[27];
    wire logic p_1_29;
    wire logic g_1_29;
    assign p_1_29 = p_0[29] & p_0[28];
    assign g_1_29 = (p_0[29] & g_0[28]) | g_0[29];
    wire logic p_1_31;
    wire logic g_1_31;
    assign p_1_31 = p_0[31] & p_0[30];
    assign g_1_31 = (p_0[31] & g_0[30]) | g_0[31];
    wire logic p_1_33;
    wire logic g_1_33;
    assign p_1_33 = p_0[33] & p_0[32];
    assign g_1_33 = (p_0[33] & g_0[32]) | g_0[33];
    wire logic p_1_35;
    wire logic g_1_35;
    assign p_1_35 = p_0[35] & p_0[34];
    assign g_1_35 = (p_0[35] & g_0[34]) | g_0[35];
    wire logic p_1_37;
    wire logic g_1_37;
    assign p_1_37 = p_0[37] & p_0[36];
    assign g_1_37 = (p_0[37] & g_0[36]) | g_0[37];
    wire logic p_1_39;
    wire logic g_1_39;
    assign p_1_39 = p_0[39] & p_0[38];
    assign g_1_39 = (p_0[39] & g_0[38]) | g_0[39];
    wire logic p_1_41;
    wire logic g_1_41;
    assign p_1_41 = p_0[41] & p_0[40];
    assign g_1_41 = (p_0[41] & g_0[40]) | g_0[41];
    wire logic p_1_43;
    wire logic g_1_43;
    assign p_1_43 = p_0[43] & p_0[42];
    assign g_1_43 = (p_0[43] & g_0[42]) | g_0[43];
    wire logic p_1_45;
    wire logic g_1_45;
    assign p_1_45 = p_0[45] & p_0[44];
    assign g_1_45 = (p_0[45] & g_0[44]) | g_0[45];
    wire logic p_1_47;
    wire logic g_1_47;
    assign p_1_47 = p_0[47] & p_0[46];
    assign g_1_47 = (p_0[47] & g_0[46]) | g_0[47];
    wire logic p_1_49;
    wire logic g_1_49;
    assign p_1_49 = p_0[49] & p_0[48];
    assign g_1_49 = (p_0[49] & g_0[48]) | g_0[49];
    wire logic p_1_51;
    wire logic g_1_51;
    assign p_1_51 = p_0[51] & p_0[50];
    assign g_1_51 = (p_0[51] & g_0[50]) | g_0[51];
    wire logic p_1_53;
    wire logic g_1_53;
    assign p_1_53 = p_0[53] & p_0[52];
    assign g_1_53 = (p_0[53] & g_0[52]) | g_0[53];
    wire logic p_1_55;
    wire logic g_1_55;
    assign p_1_55 = p_0[55] & p_0[54];
    assign g_1_55 = (p_0[55] & g_0[54]) | g_0[55];
    wire logic p_1_57;
    wire logic g_1_57;
    assign p_1_57 = p_0[57] & p_0[56];
    assign g_1_57 = (p_0[57] & g_0[56]) | g_0[57];
    wire logic p_1_59;
    wire logic g_1_59;
    assign p_1_59 = p_0[59] & p_0[58];
    assign g_1_59 = (p_0[59] & g_0[58]) | g_0[59];
    wire logic p_1_61;
    wire logic g_1_61;
    assign p_1_61 = p_0[61] & p_0[60];
    assign g_1_61 = (p_0[61] & g_0[60]) | g_0[61];
    wire logic p_1_63;
    wire logic g_1_63;
    assign p_1_63 = p_0[63] & p_0[62];
    assign g_1_63 = (p_0[63] & g_0[62]) | g_0[63];
    
    // HC stage 2
    wire logic p_2_3;
    wire logic g_2_3;
    assign p_2_3 = p_1_3 & p_1_1;
    assign g_2_3 = (p_1_3 & g_1_1) | g_1_3;
    wire logic p_2_5;
    wire logic g_2_5;
    assign p_2_5 = p_1_5 & p_1_3;
    assign g_2_5 = (p_1_5 & g_1_3) | g_1_5;
    wire logic p_2_7;
    wire logic g_2_7;
    assign p_2_7 = p_1_7 & p_1_5;
    assign g_2_7 = (p_1_7 & g_1_5) | g_1_7;
    wire logic p_2_9;
    wire logic g_2_9;
    assign p_2_9 = p_1_9 & p_1_7;
    assign g_2_9 = (p_1_9 & g_1_7) | g_1_9;
    wire logic p_2_11;
    wire logic g_2_11;
    assign p_2_11 = p_1_11 & p_1_9;
    assign g_2_11 = (p_1_11 & g_1_9) | g_1_11;
    wire logic p_2_13;
    wire logic g_2_13;
    assign p_2_13 = p_1_13 & p_1_11;
    assign g_2_13 = (p_1_13 & g_1_11) | g_1_13;
    wire logic p_2_15;
    wire logic g_2_15;
    assign p_2_15 = p_1_15 & p_1_13;
    assign g_2_15 = (p_1_15 & g_1_13) | g_1_15;
    wire logic p_2_17;
    wire logic g_2_17;
    assign p_2_17 = p_1_17 & p_1_15;
    assign g_2_17 = (p_1_17 & g_1_15) | g_1_17;
    wire logic p_2_19;
    wire logic g_2_19;
    assign p_2_19 = p_1_19 & p_1_17;
    assign g_2_19 = (p_1_19 & g_1_17) | g_1_19;
    wire logic p_2_21;
    wire logic g_2_21;
    assign p_2_21 = p_1_21 & p_1_19;
    assign g_2_21 = (p_1_21 & g_1_19) | g_1_21;
    wire logic p_2_23;
    wire logic g_2_23;
    assign p_2_23 = p_1_23 & p_1_21;
    assign g_2_23 = (p_1_23 & g_1_21) | g_1_23;
    wire logic p_2_25;
    wire logic g_2_25;
    assign p_2_25 = p_1_25 & p_1_23;
    assign g_2_25 = (p_1_25 & g_1_23) | g_1_25;
    wire logic p_2_27;
    wire logic g_2_27;
    assign p_2_27 = p_1_27 & p_1_25;
    assign g_2_27 = (p_1_27 & g_1_25) | g_1_27;
    wire logic p_2_29;
    wire logic g_2_29;
    assign p_2_29 = p_1_29 & p_1_27;
    assign g_2_29 = (p_1_29 & g_1_27) | g_1_29;
    wire logic p_2_31;
    wire logic g_2_31;
    assign p_2_31 = p_1_31 & p_1_29;
    assign g_2_31 = (p_1_31 & g_1_29) | g_1_31;
    wire logic p_2_33;
    wire logic g_2_33;
    assign p_2_33 = p_1_33 & p_1_31;
    assign g_2_33 = (p_1_33 & g_1_31) | g_1_33;
    wire logic p_2_35;
    wire logic g_2_35;
    assign p_2_35 = p_1_35 & p_1_33;
    assign g_2_35 = (p_1_35 & g_1_33) | g_1_35;
    wire logic p_2_37;
    wire logic g_2_37;
    assign p_2_37 = p_1_37 & p_1_35;
    assign g_2_37 = (p_1_37 & g_1_35) | g_1_37;
    wire logic p_2_39;
    wire logic g_2_39;
    assign p_2_39 = p_1_39 & p_1_37;
    assign g_2_39 = (p_1_39 & g_1_37) | g_1_39;
    wire logic p_2_41;
    wire logic g_2_41;
    assign p_2_41 = p_1_41 & p_1_39;
    assign g_2_41 = (p_1_41 & g_1_39) | g_1_41;
    wire logic p_2_43;
    wire logic g_2_43;
    assign p_2_43 = p_1_43 & p_1_41;
    assign g_2_43 = (p_1_43 & g_1_41) | g_1_43;
    wire logic p_2_45;
    wire logic g_2_45;
    assign p_2_45 = p_1_45 & p_1_43;
    assign g_2_45 = (p_1_45 & g_1_43) | g_1_45;
    wire logic p_2_47;
    wire logic g_2_47;
    assign p_2_47 = p_1_47 & p_1_45;
    assign g_2_47 = (p_1_47 & g_1_45) | g_1_47;
    wire logic p_2_49;
    wire logic g_2_49;
    assign p_2_49 = p_1_49 & p_1_47;
    assign g_2_49 = (p_1_49 & g_1_47) | g_1_49;
    wire logic p_2_51;
    wire logic g_2_51;
    assign p_2_51 = p_1_51 & p_1_49;
    assign g_2_51 = (p_1_51 & g_1_49) | g_1_51;
    wire logic p_2_53;
    wire logic g_2_53;
    assign p_2_53 = p_1_53 & p_1_51;
    assign g_2_53 = (p_1_53 & g_1_51) | g_1_53;
    wire logic p_2_55;
    wire logic g_2_55;
    assign p_2_55 = p_1_55 & p_1_53;
    assign g_2_55 = (p_1_55 & g_1_53) | g_1_55;
    wire logic p_2_57;
    wire logic g_2_57;
    assign p_2_57 = p_1_57 & p_1_55;
    assign g_2_57 = (p_1_57 & g_1_55) | g_1_57;
    wire logic p_2_59;
    wire logic g_2_59;
    assign p_2_59 = p_1_59 & p_1_57;
    assign g_2_59 = (p_1_59 & g_1_57) | g_1_59;
    wire logic p_2_61;
    wire logic g_2_61;
    assign p_2_61 = p_1_61 & p_1_59;
    assign g_2_61 = (p_1_61 & g_1_59) | g_1_61;
    wire logic p_2_63;
    wire logic g_2_63;
    assign p_2_63 = p_1_63 & p_1_61;
    assign g_2_63 = (p_1_63 & g_1_61) | g_1_63;
    
    // HC stage 3
    wire logic p_3_5;
    wire logic g_3_5;
    assign p_3_5 = p_2_5 & p_1_1;
    assign g_3_5 = (p_2_5 & g_1_1) | g_2_5;
    wire logic p_3_7;
    wire logic g_3_7;
    assign p_3_7 = p_2_7 & p_2_3;
    assign g_3_7 = (p_2_7 & g_2_3) | g_2_7;
    wire logic p_3_9;
    wire logic g_3_9;
    assign p_3_9 = p_2_9 & p_2_5;
    assign g_3_9 = (p_2_9 & g_2_5) | g_2_9;
    wire logic p_3_11;
    wire logic g_3_11;
    assign p_3_11 = p_2_11 & p_2_7;
    assign g_3_11 = (p_2_11 & g_2_7) | g_2_11;
    wire logic p_3_13;
    wire logic g_3_13;
    assign p_3_13 = p_2_13 & p_2_9;
    assign g_3_13 = (p_2_13 & g_2_9) | g_2_13;
    wire logic p_3_15;
    wire logic g_3_15;
    assign p_3_15 = p_2_15 & p_2_11;
    assign g_3_15 = (p_2_15 & g_2_11) | g_2_15;
    wire logic p_3_17;
    wire logic g_3_17;
    assign p_3_17 = p_2_17 & p_2_13;
    assign g_3_17 = (p_2_17 & g_2_13) | g_2_17;
    wire logic p_3_19;
    wire logic g_3_19;
    assign p_3_19 = p_2_19 & p_2_15;
    assign g_3_19 = (p_2_19 & g_2_15) | g_2_19;
    wire logic p_3_21;
    wire logic g_3_21;
    assign p_3_21 = p_2_21 & p_2_17;
    assign g_3_21 = (p_2_21 & g_2_17) | g_2_21;
    wire logic p_3_23;
    wire logic g_3_23;
    assign p_3_23 = p_2_23 & p_2_19;
    assign g_3_23 = (p_2_23 & g_2_19) | g_2_23;
    wire logic p_3_25;
    wire logic g_3_25;
    assign p_3_25 = p_2_25 & p_2_21;
    assign g_3_25 = (p_2_25 & g_2_21) | g_2_25;
    wire logic p_3_27;
    wire logic g_3_27;
    assign p_3_27 = p_2_27 & p_2_23;
    assign g_3_27 = (p_2_27 & g_2_23) | g_2_27;
    wire logic p_3_29;
    wire logic g_3_29;
    assign p_3_29 = p_2_29 & p_2_25;
    assign g_3_29 = (p_2_29 & g_2_25) | g_2_29;
    wire logic p_3_31;
    wire logic g_3_31;
    assign p_3_31 = p_2_31 & p_2_27;
    assign g_3_31 = (p_2_31 & g_2_27) | g_2_31;
    wire logic p_3_33;
    wire logic g_3_33;
    assign p_3_33 = p_2_33 & p_2_29;
    assign g_3_33 = (p_2_33 & g_2_29) | g_2_33;
    wire logic p_3_35;
    wire logic g_3_35;
    assign p_3_35 = p_2_35 & p_2_31;
    assign g_3_35 = (p_2_35 & g_2_31) | g_2_35;
    wire logic p_3_37;
    wire logic g_3_37;
    assign p_3_37 = p_2_37 & p_2_33;
    assign g_3_37 = (p_2_37 & g_2_33) | g_2_37;
    wire logic p_3_39;
    wire logic g_3_39;
    assign p_3_39 = p_2_39 & p_2_35;
    assign g_3_39 = (p_2_39 & g_2_35) | g_2_39;
    wire logic p_3_41;
    wire logic g_3_41;
    assign p_3_41 = p_2_41 & p_2_37;
    assign g_3_41 = (p_2_41 & g_2_37) | g_2_41;
    wire logic p_3_43;
    wire logic g_3_43;
    assign p_3_43 = p_2_43 & p_2_39;
    assign g_3_43 = (p_2_43 & g_2_39) | g_2_43;
    wire logic p_3_45;
    wire logic g_3_45;
    assign p_3_45 = p_2_45 & p_2_41;
    assign g_3_45 = (p_2_45 & g_2_41) | g_2_45;
    wire logic p_3_47;
    wire logic g_3_47;
    assign p_3_47 = p_2_47 & p_2_43;
    assign g_3_47 = (p_2_47 & g_2_43) | g_2_47;
    wire logic p_3_49;
    wire logic g_3_49;
    assign p_3_49 = p_2_49 & p_2_45;
    assign g_3_49 = (p_2_49 & g_2_45) | g_2_49;
    wire logic p_3_51;
    wire logic g_3_51;
    assign p_3_51 = p_2_51 & p_2_47;
    assign g_3_51 = (p_2_51 & g_2_47) | g_2_51;
    wire logic p_3_53;
    wire logic g_3_53;
    assign p_3_53 = p_2_53 & p_2_49;
    assign g_3_53 = (p_2_53 & g_2_49) | g_2_53;
    wire logic p_3_55;
    wire logic g_3_55;
    assign p_3_55 = p_2_55 & p_2_51;
    assign g_3_55 = (p_2_55 & g_2_51) | g_2_55;
    wire logic p_3_57;
    wire logic g_3_57;
    assign p_3_57 = p_2_57 & p_2_53;
    assign g_3_57 = (p_2_57 & g_2_53) | g_2_57;
    wire logic p_3_59;
    wire logic g_3_59;
    assign p_3_59 = p_2_59 & p_2_55;
    assign g_3_59 = (p_2_59 & g_2_55) | g_2_59;
    wire logic p_3_61;
    wire logic g_3_61;
    assign p_3_61 = p_2_61 & p_2_57;
    assign g_3_61 = (p_2_61 & g_2_57) | g_2_61;
    wire logic p_3_63;
    wire logic g_3_63;
    assign p_3_63 = p_2_63 & p_2_59;
    assign g_3_63 = (p_2_63 & g_2_59) | g_2_63;
    
    // HC stage 4
    wire logic p_4_9;
    wire logic g_4_9;
    assign p_4_9 = p_3_9 & p_1_1;
    assign g_4_9 = (p_3_9 & g_1_1) | g_3_9;
    wire logic p_4_11;
    wire logic g_4_11;
    assign p_4_11 = p_3_11 & p_2_3;
    assign g_4_11 = (p_3_11 & g_2_3) | g_3_11;
    wire logic p_4_13;
    wire logic g_4_13;
    assign p_4_13 = p_3_13 & p_3_5;
    assign g_4_13 = (p_3_13 & g_3_5) | g_3_13;
    wire logic p_4_15;
    wire logic g_4_15;
    assign p_4_15 = p_3_15 & p_3_7;
    assign g_4_15 = (p_3_15 & g_3_7) | g_3_15;
    wire logic p_4_17;
    wire logic g_4_17;
    assign p_4_17 = p_3_17 & p_3_9;
    assign g_4_17 = (p_3_17 & g_3_9) | g_3_17;
    wire logic p_4_19;
    wire logic g_4_19;
    assign p_4_19 = p_3_19 & p_3_11;
    assign g_4_19 = (p_3_19 & g_3_11) | g_3_19;
    wire logic p_4_21;
    wire logic g_4_21;
    assign p_4_21 = p_3_21 & p_3_13;
    assign g_4_21 = (p_3_21 & g_3_13) | g_3_21;
    wire logic p_4_23;
    wire logic g_4_23;
    assign p_4_23 = p_3_23 & p_3_15;
    assign g_4_23 = (p_3_23 & g_3_15) | g_3_23;
    wire logic p_4_25;
    wire logic g_4_25;
    assign p_4_25 = p_3_25 & p_3_17;
    assign g_4_25 = (p_3_25 & g_3_17) | g_3_25;
    wire logic p_4_27;
    wire logic g_4_27;
    assign p_4_27 = p_3_27 & p_3_19;
    assign g_4_27 = (p_3_27 & g_3_19) | g_3_27;
    wire logic p_4_29;
    wire logic g_4_29;
    assign p_4_29 = p_3_29 & p_3_21;
    assign g_4_29 = (p_3_29 & g_3_21) | g_3_29;
    wire logic p_4_31;
    wire logic g_4_31;
    assign p_4_31 = p_3_31 & p_3_23;
    assign g_4_31 = (p_3_31 & g_3_23) | g_3_31;
    wire logic p_4_33;
    wire logic g_4_33;
    assign p_4_33 = p_3_33 & p_3_25;
    assign g_4_33 = (p_3_33 & g_3_25) | g_3_33;
    wire logic p_4_35;
    wire logic g_4_35;
    assign p_4_35 = p_3_35 & p_3_27;
    assign g_4_35 = (p_3_35 & g_3_27) | g_3_35;
    wire logic p_4_37;
    wire logic g_4_37;
    assign p_4_37 = p_3_37 & p_3_29;
    assign g_4_37 = (p_3_37 & g_3_29) | g_3_37;
    wire logic p_4_39;
    wire logic g_4_39;
    assign p_4_39 = p_3_39 & p_3_31;
    assign g_4_39 = (p_3_39 & g_3_31) | g_3_39;
    wire logic p_4_41;
    wire logic g_4_41;
    assign p_4_41 = p_3_41 & p_3_33;
    assign g_4_41 = (p_3_41 & g_3_33) | g_3_41;
    wire logic p_4_43;
    wire logic g_4_43;
    assign p_4_43 = p_3_43 & p_3_35;
    assign g_4_43 = (p_3_43 & g_3_35) | g_3_43;
    wire logic p_4_45;
    wire logic g_4_45;
    assign p_4_45 = p_3_45 & p_3_37;
    assign g_4_45 = (p_3_45 & g_3_37) | g_3_45;
    wire logic p_4_47;
    wire logic g_4_47;
    assign p_4_47 = p_3_47 & p_3_39;
    assign g_4_47 = (p_3_47 & g_3_39) | g_3_47;
    wire logic p_4_49;
    wire logic g_4_49;
    assign p_4_49 = p_3_49 & p_3_41;
    assign g_4_49 = (p_3_49 & g_3_41) | g_3_49;
    wire logic p_4_51;
    wire logic g_4_51;
    assign p_4_51 = p_3_51 & p_3_43;
    assign g_4_51 = (p_3_51 & g_3_43) | g_3_51;
    wire logic p_4_53;
    wire logic g_4_53;
    assign p_4_53 = p_3_53 & p_3_45;
    assign g_4_53 = (p_3_53 & g_3_45) | g_3_53;
    wire logic p_4_55;
    wire logic g_4_55;
    assign p_4_55 = p_3_55 & p_3_47;
    assign g_4_55 = (p_3_55 & g_3_47) | g_3_55;
    wire logic p_4_57;
    wire logic g_4_57;
    assign p_4_57 = p_3_57 & p_3_49;
    assign g_4_57 = (p_3_57 & g_3_49) | g_3_57;
    wire logic p_4_59;
    wire logic g_4_59;
    assign p_4_59 = p_3_59 & p_3_51;
    assign g_4_59 = (p_3_59 & g_3_51) | g_3_59;
    wire logic p_4_61;
    wire logic g_4_61;
    assign p_4_61 = p_3_61 & p_3_53;
    assign g_4_61 = (p_3_61 & g_3_53) | g_3_61;
    wire logic p_4_63;
    wire logic g_4_63;
    assign p_4_63 = p_3_63 & p_3_55;
    assign g_4_63 = (p_3_63 & g_3_55) | g_3_63;
    
    // HC stage 5
    wire logic p_5_17;
    wire logic g_5_17;
    assign p_5_17 = p_4_17 & p_1_1;
    assign g_5_17 = (p_4_17 & g_1_1) | g_4_17;
    wire logic p_5_19;
    wire logic g_5_19;
    assign p_5_19 = p_4_19 & p_2_3;
    assign g_5_19 = (p_4_19 & g_2_3) | g_4_19;
    wire logic p_5_21;
    wire logic g_5_21;
    assign p_5_21 = p_4_21 & p_3_5;
    assign g_5_21 = (p_4_21 & g_3_5) | g_4_21;
    wire logic p_5_23;
    wire logic g_5_23;
    assign p_5_23 = p_4_23 & p_3_7;
    assign g_5_23 = (p_4_23 & g_3_7) | g_4_23;
    wire logic p_5_25;
    wire logic g_5_25;
    assign p_5_25 = p_4_25 & p_4_9;
    assign g_5_25 = (p_4_25 & g_4_9) | g_4_25;
    wire logic p_5_27;
    wire logic g_5_27;
    assign p_5_27 = p_4_27 & p_4_11;
    assign g_5_27 = (p_4_27 & g_4_11) | g_4_27;
    wire logic p_5_29;
    wire logic g_5_29;
    assign p_5_29 = p_4_29 & p_4_13;
    assign g_5_29 = (p_4_29 & g_4_13) | g_4_29;
    wire logic p_5_31;
    wire logic g_5_31;
    assign p_5_31 = p_4_31 & p_4_15;
    assign g_5_31 = (p_4_31 & g_4_15) | g_4_31;
    wire logic p_5_33;
    wire logic g_5_33;
    assign p_5_33 = p_4_33 & p_4_17;
    assign g_5_33 = (p_4_33 & g_4_17) | g_4_33;
    wire logic p_5_35;
    wire logic g_5_35;
    assign p_5_35 = p_4_35 & p_4_19;
    assign g_5_35 = (p_4_35 & g_4_19) | g_4_35;
    wire logic p_5_37;
    wire logic g_5_37;
    assign p_5_37 = p_4_37 & p_4_21;
    assign g_5_37 = (p_4_37 & g_4_21) | g_4_37;
    wire logic p_5_39;
    wire logic g_5_39;
    assign p_5_39 = p_4_39 & p_4_23;
    assign g_5_39 = (p_4_39 & g_4_23) | g_4_39;
    wire logic p_5_41;
    wire logic g_5_41;
    assign p_5_41 = p_4_41 & p_4_25;
    assign g_5_41 = (p_4_41 & g_4_25) | g_4_41;
    wire logic p_5_43;
    wire logic g_5_43;
    assign p_5_43 = p_4_43 & p_4_27;
    assign g_5_43 = (p_4_43 & g_4_27) | g_4_43;
    wire logic p_5_45;
    wire logic g_5_45;
    assign p_5_45 = p_4_45 & p_4_29;
    assign g_5_45 = (p_4_45 & g_4_29) | g_4_45;
    wire logic p_5_47;
    wire logic g_5_47;
    assign p_5_47 = p_4_47 & p_4_31;
    assign g_5_47 = (p_4_47 & g_4_31) | g_4_47;
    wire logic p_5_49;
    wire logic g_5_49;
    assign p_5_49 = p_4_49 & p_4_33;
    assign g_5_49 = (p_4_49 & g_4_33) | g_4_49;
    wire logic p_5_51;
    wire logic g_5_51;
    assign p_5_51 = p_4_51 & p_4_35;
    assign g_5_51 = (p_4_51 & g_4_35) | g_4_51;
    wire logic p_5_53;
    wire logic g_5_53;
    assign p_5_53 = p_4_53 & p_4_37;
    assign g_5_53 = (p_4_53 & g_4_37) | g_4_53;
    wire logic p_5_55;
    wire logic g_5_55;
    assign p_5_55 = p_4_55 & p_4_39;
    assign g_5_55 = (p_4_55 & g_4_39) | g_4_55;
    wire logic p_5_57;
    wire logic g_5_57;
    assign p_5_57 = p_4_57 & p_4_41;
    assign g_5_57 = (p_4_57 & g_4_41) | g_4_57;
    wire logic p_5_59;
    wire logic g_5_59;
    assign p_5_59 = p_4_59 & p_4_43;
    assign g_5_59 = (p_4_59 & g_4_43) | g_4_59;
    wire logic p_5_61;
    wire logic g_5_61;
    assign p_5_61 = p_4_61 & p_4_45;
    assign g_5_61 = (p_4_61 & g_4_45) | g_4_61;
    wire logic p_5_63;
    wire logic g_5_63;
    assign p_5_63 = p_4_63 & p_4_47;
    assign g_5_63 = (p_4_63 & g_4_47) | g_4_63;
    
    // HC stage 6
    wire logic p_6_33;
    wire logic g_6_33;
    assign p_6_33 = p_5_33 & p_1_1;
    assign g_6_33 = (p_5_33 & g_1_1) | g_5_33;
    wire logic p_6_35;
    wire logic g_6_35;
    assign p_6_35 = p_5_35 & p_2_3;
    assign g_6_35 = (p_5_35 & g_2_3) | g_5_35;
    wire logic p_6_37;
    wire logic g_6_37;
    assign p_6_37 = p_5_37 & p_3_5;
    assign g_6_37 = (p_5_37 & g_3_5) | g_5_37;
    wire logic p_6_39;
    wire logic g_6_39;
    assign p_6_39 = p_5_39 & p_3_7;
    assign g_6_39 = (p_5_39 & g_3_7) | g_5_39;
    wire logic p_6_41;
    wire logic g_6_41;
    assign p_6_41 = p_5_41 & p_4_9;
    assign g_6_41 = (p_5_41 & g_4_9) | g_5_41;
    wire logic p_6_43;
    wire logic g_6_43;
    assign p_6_43 = p_5_43 & p_4_11;
    assign g_6_43 = (p_5_43 & g_4_11) | g_5_43;
    wire logic p_6_45;
    wire logic g_6_45;
    assign p_6_45 = p_5_45 & p_4_13;
    assign g_6_45 = (p_5_45 & g_4_13) | g_5_45;
    wire logic p_6_47;
    wire logic g_6_47;
    assign p_6_47 = p_5_47 & p_4_15;
    assign g_6_47 = (p_5_47 & g_4_15) | g_5_47;
    wire logic p_6_49;
    wire logic g_6_49;
    assign p_6_49 = p_5_49 & p_5_17;
    assign g_6_49 = (p_5_49 & g_5_17) | g_5_49;
    wire logic p_6_51;
    wire logic g_6_51;
    assign p_6_51 = p_5_51 & p_5_19;
    assign g_6_51 = (p_5_51 & g_5_19) | g_5_51;
    wire logic p_6_53;
    wire logic g_6_53;
    assign p_6_53 = p_5_53 & p_5_21;
    assign g_6_53 = (p_5_53 & g_5_21) | g_5_53;
    wire logic p_6_55;
    wire logic g_6_55;
    assign p_6_55 = p_5_55 & p_5_23;
    assign g_6_55 = (p_5_55 & g_5_23) | g_5_55;
    wire logic p_6_57;
    wire logic g_6_57;
    assign p_6_57 = p_5_57 & p_5_25;
    assign g_6_57 = (p_5_57 & g_5_25) | g_5_57;
    wire logic p_6_59;
    wire logic g_6_59;
    assign p_6_59 = p_5_59 & p_5_27;
    assign g_6_59 = (p_5_59 & g_5_27) | g_5_59;
    wire logic p_6_61;
    wire logic g_6_61;
    assign p_6_61 = p_5_61 & p_5_29;
    assign g_6_61 = (p_5_61 & g_5_29) | g_5_61;
    wire logic p_6_63;
    wire logic g_6_63;
    assign p_6_63 = p_5_63 & p_5_31;
    assign g_6_63 = (p_5_63 & g_5_31) | g_5_63;
    
    // HC stage 7
    wire logic p_7_2;
    wire logic g_7_2;
    assign p_7_2 = p_0[2] & p_1_1;
    assign g_7_2 = (p_0[2] & g_1_1) | g_0[2];
    wire logic p_7_4;
    wire logic g_7_4;
    assign p_7_4 = p_0[4] & p_2_3;
    assign g_7_4 = (p_0[4] & g_2_3) | g_0[4];
    wire logic p_7_6;
    wire logic g_7_6;
    assign p_7_6 = p_0[6] & p_3_5;
    assign g_7_6 = (p_0[6] & g_3_5) | g_0[6];
    wire logic p_7_8;
    wire logic g_7_8;
    assign p_7_8 = p_0[8] & p_3_7;
    assign g_7_8 = (p_0[8] & g_3_7) | g_0[8];
    wire logic p_7_10;
    wire logic g_7_10;
    assign p_7_10 = p_0[10] & p_4_9;
    assign g_7_10 = (p_0[10] & g_4_9) | g_0[10];
    wire logic p_7_12;
    wire logic g_7_12;
    assign p_7_12 = p_0[12] & p_4_11;
    assign g_7_12 = (p_0[12] & g_4_11) | g_0[12];
    wire logic p_7_14;
    wire logic g_7_14;
    assign p_7_14 = p_0[14] & p_4_13;
    assign g_7_14 = (p_0[14] & g_4_13) | g_0[14];
    wire logic p_7_16;
    wire logic g_7_16;
    assign p_7_16 = p_0[16] & p_4_15;
    assign g_7_16 = (p_0[16] & g_4_15) | g_0[16];
    wire logic p_7_18;
    wire logic g_7_18;
    assign p_7_18 = p_0[18] & p_5_17;
    assign g_7_18 = (p_0[18] & g_5_17) | g_0[18];
    wire logic p_7_20;
    wire logic g_7_20;
    assign p_7_20 = p_0[20] & p_5_19;
    assign g_7_20 = (p_0[20] & g_5_19) | g_0[20];
    wire logic p_7_22;
    wire logic g_7_22;
    assign p_7_22 = p_0[22] & p_5_21;
    assign g_7_22 = (p_0[22] & g_5_21) | g_0[22];
    wire logic p_7_24;
    wire logic g_7_24;
    assign p_7_24 = p_0[24] & p_5_23;
    assign g_7_24 = (p_0[24] & g_5_23) | g_0[24];
    wire logic p_7_26;
    wire logic g_7_26;
    assign p_7_26 = p_0[26] & p_5_25;
    assign g_7_26 = (p_0[26] & g_5_25) | g_0[26];
    wire logic p_7_28;
    wire logic g_7_28;
    assign p_7_28 = p_0[28] & p_5_27;
    assign g_7_28 = (p_0[28] & g_5_27) | g_0[28];
    wire logic p_7_30;
    wire logic g_7_30;
    assign p_7_30 = p_0[30] & p_5_29;
    assign g_7_30 = (p_0[30] & g_5_29) | g_0[30];
    wire logic p_7_32;
    wire logic g_7_32;
    assign p_7_32 = p_0[32] & p_5_31;
    assign g_7_32 = (p_0[32] & g_5_31) | g_0[32];
    wire logic p_7_34;
    wire logic g_7_34;
    assign p_7_34 = p_0[34] & p_6_33;
    assign g_7_34 = (p_0[34] & g_6_33) | g_0[34];
    wire logic p_7_36;
    wire logic g_7_36;
    assign p_7_36 = p_0[36] & p_6_35;
    assign g_7_36 = (p_0[36] & g_6_35) | g_0[36];
    wire logic p_7_38;
    wire logic g_7_38;
    assign p_7_38 = p_0[38] & p_6_37;
    assign g_7_38 = (p_0[38] & g_6_37) | g_0[38];
    wire logic p_7_40;
    wire logic g_7_40;
    assign p_7_40 = p_0[40] & p_6_39;
    assign g_7_40 = (p_0[40] & g_6_39) | g_0[40];
    wire logic p_7_42;
    wire logic g_7_42;
    assign p_7_42 = p_0[42] & p_6_41;
    assign g_7_42 = (p_0[42] & g_6_41) | g_0[42];
    wire logic p_7_44;
    wire logic g_7_44;
    assign p_7_44 = p_0[44] & p_6_43;
    assign g_7_44 = (p_0[44] & g_6_43) | g_0[44];
    wire logic p_7_46;
    wire logic g_7_46;
    assign p_7_46 = p_0[46] & p_6_45;
    assign g_7_46 = (p_0[46] & g_6_45) | g_0[46];
    wire logic p_7_48;
    wire logic g_7_48;
    assign p_7_48 = p_0[48] & p_6_47;
    assign g_7_48 = (p_0[48] & g_6_47) | g_0[48];
    wire logic p_7_50;
    wire logic g_7_50;
    assign p_7_50 = p_0[50] & p_6_49;
    assign g_7_50 = (p_0[50] & g_6_49) | g_0[50];
    wire logic p_7_52;
    wire logic g_7_52;
    assign p_7_52 = p_0[52] & p_6_51;
    assign g_7_52 = (p_0[52] & g_6_51) | g_0[52];
    wire logic p_7_54;
    wire logic g_7_54;
    assign p_7_54 = p_0[54] & p_6_53;
    assign g_7_54 = (p_0[54] & g_6_53) | g_0[54];
    wire logic p_7_56;
    wire logic g_7_56;
    assign p_7_56 = p_0[56] & p_6_55;
    assign g_7_56 = (p_0[56] & g_6_55) | g_0[56];
    wire logic p_7_58;
    wire logic g_7_58;
    assign p_7_58 = p_0[58] & p_6_57;
    assign g_7_58 = (p_0[58] & g_6_57) | g_0[58];
    wire logic p_7_60;
    wire logic g_7_60;
    assign p_7_60 = p_0[60] & p_6_59;
    assign g_7_60 = (p_0[60] & g_6_59) | g_0[60];
    wire logic p_7_62;
    wire logic g_7_62;
    assign p_7_62 = p_0[62] & p_6_61;
    assign g_7_62 = (p_0[62] & g_6_61) | g_0[62];
    
    // HC postprocess 
    assign OUT[0] = p_0[0];
    assign OUT[1] = p_0[1] ^ g_0[0];
    assign OUT[2] = p_0[2] ^ g_1_1;
    assign OUT[3] = p_0[3] ^ g_7_2;
    assign OUT[4] = p_0[4] ^ g_2_3;
    assign OUT[5] = p_0[5] ^ g_7_4;
    assign OUT[6] = p_0[6] ^ g_3_5;
    assign OUT[7] = p_0[7] ^ g_7_6;
    assign OUT[8] = p_0[8] ^ g_3_7;
    assign OUT[9] = p_0[9] ^ g_7_8;
    assign OUT[10] = p_0[10] ^ g_4_9;
    assign OUT[11] = p_0[11] ^ g_7_10;
    assign OUT[12] = p_0[12] ^ g_4_11;
    assign OUT[13] = p_0[13] ^ g_7_12;
    assign OUT[14] = p_0[14] ^ g_4_13;
    assign OUT[15] = p_0[15] ^ g_7_14;
    assign OUT[16] = p_0[16] ^ g_4_15;
    assign OUT[17] = p_0[17] ^ g_7_16;
    assign OUT[18] = p_0[18] ^ g_5_17;
    assign OUT[19] = p_0[19] ^ g_7_18;
    assign OUT[20] = p_0[20] ^ g_5_19;
    assign OUT[21] = p_0[21] ^ g_7_20;
    assign OUT[22] = p_0[22] ^ g_5_21;
    assign OUT[23] = p_0[23] ^ g_7_22;
    assign OUT[24] = p_0[24] ^ g_5_23;
    assign OUT[25] = p_0[25] ^ g_7_24;
    assign OUT[26] = p_0[26] ^ g_5_25;
    assign OUT[27] = p_0[27] ^ g_7_26;
    assign OUT[28] = p_0[28] ^ g_5_27;
    assign OUT[29] = p_0[29] ^ g_7_28;
    assign OUT[30] = p_0[30] ^ g_5_29;
    assign OUT[31] = p_0[31] ^ g_7_30;
    assign OUT[32] = p_0[32] ^ g_5_31;
    assign OUT[33] = p_0[33] ^ g_7_32;
    assign OUT[34] = p_0[34] ^ g_6_33;
    assign OUT[35] = p_0[35] ^ g_7_34;
    assign OUT[36] = p_0[36] ^ g_6_35;
    assign OUT[37] = p_0[37] ^ g_7_36;
    assign OUT[38] = p_0[38] ^ g_6_37;
    assign OUT[39] = p_0[39] ^ g_7_38;
    assign OUT[40] = p_0[40] ^ g_6_39;
    assign OUT[41] = p_0[41] ^ g_7_40;
    assign OUT[42] = p_0[42] ^ g_6_41;
    assign OUT[43] = p_0[43] ^ g_7_42;
    assign OUT[44] = p_0[44] ^ g_6_43;
    assign OUT[45] = p_0[45] ^ g_7_44;
    assign OUT[46] = p_0[46] ^ g_6_45;
    assign OUT[47] = p_0[47] ^ g_7_46;
    assign OUT[48] = p_0[48] ^ g_6_47;
    assign OUT[49] = p_0[49] ^ g_7_48;
    assign OUT[50] = p_0[50] ^ g_6_49;
    assign OUT[51] = p_0[51] ^ g_7_50;
    assign OUT[52] = p_0[52] ^ g_6_51;
    assign OUT[53] = p_0[53] ^ g_7_52;
    assign OUT[54] = p_0[54] ^ g_6_53;
    assign OUT[55] = p_0[55] ^ g_7_54;
    assign OUT[56] = p_0[56] ^ g_6_55;
    assign OUT[57] = p_0[57] ^ g_7_56;
    assign OUT[58] = p_0[58] ^ g_6_57;
    assign OUT[59] = p_0[59] ^ g_7_58;
    assign OUT[60] = p_0[60] ^ g_6_59;
    assign OUT[61] = p_0[61] ^ g_7_60;
    assign OUT[62] = p_0[62] ^ g_6_61;
    assign OUT[63] = p_0[63] ^ g_7_62;
    assign OUT[64] = g_6_63;
endmodule

module HC_64_spec (
        input logic [63:0] IN1,
        input logic [63:0] IN2,
        output logic adder_correct,
        output logic [64:0] spec_res);
    
    assign spec_res = IN1 + IN2;
    wire [64:0] adder_res;
    HC_64 adder(IN1, IN2, adder_res);
    assign adder_correct = ((spec_res == adder_res) ? 1 : 0);
    
endmodule



module ha (
        input logic a,
        input logic b,
        output logic s,
        output logic c);
    
    assign s = a ^ b;
    assign c = a & b;
endmodule



module fa (
        input logic x,
        input logic y,
        input logic z,
        output logic s,
        output logic c);
    
    assign s = x ^ y ^ z;
    assign c = (x & y) | (x & z) | (y & z);
endmodule

module Four2Two 
        #(parameter WIDTH=1) (
        input logic [WIDTH-1:0] in1,
        input logic [WIDTH-1:0] in2,
        input logic [WIDTH-1:0] in3,
        input logic [WIDTH-1:0] in4,
        input logic cin,
        output logic [WIDTH-1:0] sum,
        output logic [WIDTH-1:0] carry,
        output logic cout);
    
    wire logic [WIDTH:0] temp1;
    assign temp1 = {((in1 ^ in2)&in3 | in1 & ~(in1^in2)),cin};
    assign sum = ((in1 ^ in2) ^ in3 ^ in4) ^ temp1[WIDTH-1:0];
    assign carry = ((in1 ^ in2) ^ in3 ^ in4) & temp1[WIDTH-1:0] | in4 & ~((in1 ^ in2) ^ in3 ^ in4);
    assign cout = temp1[WIDTH];
endmodule




