; Note: The license below is based on the template at:
; http://opensource.org/licenses/BSD-3-Clause

; Copyright (C) 2023 Regents of the University of Texas
; All rights reserved.

; Redistribution and use in source and binary forms, with or without
; modification, are permitted provided that the following conditions are
; met:

; o Redistributions of source code must retain the above copyright
;   notice, this list of conditions and the following disclaimer.

; o Redistributions in binary form must reproduce the above copyright
;   notice, this list of conditions and the following disclaimer in the
;   documentation and/or other materials provided with the distribution.

; o Neither the name of the copyright holders nor the names of its
;   contributors may be used to endorse or promote products derived
;   from this software without specific prior written permission.

; THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
; "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
; LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
; A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
; HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
; SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
; LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
; DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
; THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
; (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
; OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

; Original Author(s):
; Mertcan Temel         <mert@utexas.edu>

(in-package "RP")

(include-book "projects/rp-rewriter/lib/mult3/top" :dir :system)

;; Increase rewriter step limit. Usually only necessary for very large multipliers. 
(rp::set-rw-step-limit (expt 2 30))
(rp::set-rp-backchain-limit 10000)

;; increase the maximum memory to 16GB that triggers garbage collection (only for CCL) 
(value-triple (acl2::set-max-mem (* 16 (expt 2 30))))




;; Example 1
;; A 64x64-bit multiplier with 128-bit result. Signed, booth radix4 encoding.
;; Dadda summation tree and Brent-Kung final stage adder.
(parse-and-create-svtv :file "example-verilog-files/DT_SB4_BK_64x64_multgen.sv"
                       :topmodule "DT_SB4_BK_64x64_spec"
                       :save-to-file "parsed/"
                       :name DT-SB4-BK-64x64)


;; Example 2
;; Same multiplier but a bug is inserted 
(parse-and-create-svtv :file "example-verilog-files/buggy-DT_SB4_BK_64x64.sv"
                       :topmodule "DT_SB4_BK_64x64_spec"
                       :save-to-file "parsed/"
                       :name buggy-DT-SB4-BK-64x64)


;; Example 3
;; A Multiply-add module. Performs 32x32-bit multiplication and adds the
;; result to a 32-bit number.  It usses Dadda tree, Han-carlson vector adder,
;; and Booth radix-8 encoding.
(parse-and-create-svtv :file "example-verilog-files/MAC_DT_SB8_HC_32x32_plus_32_63to0_multgen.sv"
                       :topmodule "MAC_DT_SB8_HC_32x32_plus_32_63to0_spec"
                       :save-to-file "parsed/"
                       :name MAC_DT_SB8_HC_32x32_plus_32_63to0)


;; Example 4
;; A randomly generated Dot-product accumulate design. Performs 6 11x9-bit
;; multiplications and adds the result to a 23-bit number. Then only the middle
;; portion of the result is outputted (from bit 22 to bit 3, inclusive).
(parse-and-create-svtv :file "example-verilog-files/DOT_Product_WT_SB4_JSkCond_6_11x9_plus_23_22to3_multgen.sv"
                       :topmodule "DOT_Product_WT_SB4_JSkCond_6_11x9_plus_23_22to3_spec"
                       :save-to-file "parsed/"
                       :name DOT_Product_WT_SB4_JSkCond_6_11x9_plus_23_22to3)


